@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -52,16 +52,14 @@ bool
 Bhv_ScanField::execute( PlayerAgent * agent )
 {
     dlog.addText( Logger::ACTION,
-                  "%s:%d: Bhf_ScanField"
-                  ,__FILE__, __LINE__ );
+                  __FILE__": Bhf_ScanField" );
 
     const WorldModel & wm = agent->world();
 
     if ( ! wm.self().posValid() )
     {
         dlog.addText( Logger::ACTION,
-                      "%s:%d: invalid my pos"
-                      ,__FILE__, __LINE__ );
+                      __FILE__": invalid my pos" );
         agent->doTurn( 60.0 );
         agent->setNeckAction( new Neck_TurnToRelative( 0.0 ) );
         return true;
@@ -69,30 +67,40 @@ Bhv_ScanField::execute( PlayerAgent * agent )
 
     if ( wm.ball().posValid() )
     {
-        agent->doTurn( 90.0 );
-        agent->setNeckAction( new Neck_ScanField() );
+//         agent->doTurn( 90.0 );
+//         agent->setNeckAction( new Neck_ScanField() );
+        scanAllField( agent );
         return true;
     }
 
-    /////////////////////////////////////////////////////////////
+    findBall( agent );
+    return true;
+}
 
-    // ball search mode
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+Bhv_ScanField::findBall( PlayerAgent * agent )
+{
+    const WorldModel & wm = agent->world();
 
     if ( wm.seeTime() != wm.time() )
     {
         dlog.addText( Logger::ACTION,
-                      "%s:%d: no current see info"
-                      ,__FILE__, __LINE__ );
+                      __FILE__": (findBall) no current see info" );
         agent->doTurn( 0.0 );
         agent->setNeckAction( new Neck_TurnToRelative( wm.self().neck() ) );
-        return false;
+        return;
     }
 
     // change to WIDE view mode
     // and only face to last ball seen pos or reverse side
-    if ( wm.self().viewWidth() != ViewWidth::WIDE )
+    if ( agent->effector().queuedNextViewWidth().type() != ViewWidth::WIDE )
     {
-        agent->doChangeView( ViewWidth::WIDE );
+        //agent->doChangeView( ViewWidth::WIDE );
+        agent->setViewAction( new View_Wide() );
     }
 
     Vector2D mynext
@@ -119,12 +127,46 @@ Bhv_ScanField::execute( PlayerAgent * agent )
         + Vector2D::polar2vector( 10.0, face_angle );
 
     dlog.addText( Logger::ACTION,
-                  "%s:%d: ball search. lost_count=%d, search_flag=%d, angle=%f"
-                  ,__FILE__, __LINE__,
+                  __FILE__": (findBall) lost_count=%d, search_flag=%d, angle=%.1f",
                   wm.ball().lostCount(),
                   search_flag,
                   face_angle.degree() );
-    return Bhv_NeckBodyToPoint( face_point ).execute( agent );
+    Bhv_NeckBodyToPoint( face_point ).execute( agent );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+Bhv_ScanField::scanAllField( PlayerAgent * agent )
+{
+    const WorldModel & wm = agent->world();
+
+    if ( wm.seeTime() != wm.time() )
+    {
+        dlog.addText( Logger::ACTION,
+                      __FILE__":scanAllField) no current see info" );
+        agent->doTurn( 0.0 );
+        agent->setNeckAction( new Neck_TurnToRelative( wm.self().neck() ) );
+        return;
+    }
+
+    if ( agent->effector().queuedNextViewWidth().type() != ViewWidth::WIDE )
+    {
+        agent->setViewAction( new View_Wide() );
+
+        dlog.addText( Logger::ACTION,
+                      __FILE__": (scanAllField)" );
+    }
+
+    AngleDeg turn_moment
+        = ( wm.self().viewWidth().width()
+            + agent->effector().queuedNextViewWidth().width() )
+        * 0.5;
+
+    agent->doTurn( turn_moment ); // current width
+    agent->setNeckAction( new Neck_TurnToRelative( wm.self().neck() ) );
 }
 
 }
