@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -528,7 +528,7 @@ Body_HoldBall2008::evaluateKeepPoint( const WorldModel & wm,
                                       const Vector2D & keep_point )
 {
     static const Rect2D penalty_area( Vector2D( ServerParam::i().theirPenaltyAreaLineX(),
-                                                ServerParam::i().penaltyAreaHalfWidth() ),
+                                                - ServerParam::i().penaltyAreaHalfWidth() ),
                                       Size2D( ServerParam::i().penaltyAreaLength(),
                                               ServerParam::i().penaltyAreaWidth() ) );
     static const double consider_dist = ( ServerParam::i().tackleDist()
@@ -708,6 +708,23 @@ Body_HoldBall2008::evaluateKeepPoint( const WorldModel & wm,
         }
     }
 
+#if 1
+    double ball_move_dist = ( keep_point - wm.ball().pos() ).r();
+    if ( ball_move_dist > wm.self().playerType().kickableArea() * 1.6 )
+    {
+        double next_ball_dist = my_next.dist( keep_point );
+        double threshold = wm.self().playerType().kickableArea() - 0.4;
+        double rate = 1.0 - 0.5 * std::max( 0.0, ( next_ball_dist - threshold ) / 0.4 );
+        score *= rate;
+
+#ifdef DEBUG
+        dlog.addText( Logger::ACTION,
+                      "__ applied keep distance threshold. ball_dist=%.3f thr=%.3f rate=%f"
+                      next_ball_dist, threshold, rate );
+#endif
+    }
+#endif
+
     return score;
 }
 
