@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -80,14 +80,17 @@ private:
 
 
     AngleDeg calcAngle( const PlayerAgent * agent );
-    AngleDeg calcAngleDefault( const PlayerAgent * agent );
+    AngleDeg calcAngleDefault( const PlayerAgent * agent,
+                               const bool consider_pitch );
 
     /*!
       \retval -360.0 if no solution.
      */
     double calcAngleForWidePitchEdge( const PlayerAgent * agent );
 
-    double calcAnglePlayerCount( const PlayerAgent * agent );
+    double calcAnglePlayerCount( const PlayerAgent * agent,
+                                 const bool with_narrow_mode,
+                                 const bool with_all_players );
 
 };
 
