@@ -14,7 +14,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -38,7 +38,6 @@
 #include "intention_dribble2007.h"
 
 #include <rcsc/action/basic_actions.h>
-#include <rcsc/action/body_go_to_point.h>
 #include <rcsc/action/body_hold_ball.h>
 #include "body_intercept2007.h"
 #include <rcsc/action/body_kick_to_relative.h>
@@ -588,7 +587,7 @@ Body_Dribble2007::doKickTurnsDash( PlayerAgent * agent,
     // firstvel = travel / (1 + dec + dec^2 + ...)
     const double term
         = ( 1.0 - std::pow( ServerParam::i().ballDecay(),
-                            static_cast< double >( n_turn + 2 ) ) )
+                            n_turn + 2 ) )
         / ( 1.0 - ServerParam::i().ballDecay() );
     const Vector2D required_first_vel
         = ( required_ball_rel_pos - wm.ball().rpos() ) / term;
@@ -918,11 +917,11 @@ Body_Dribble2007::doKickDashes( PlayerAgent * agent,
             y_dist = std::min( y_dist, cur_ball_rel.absY() );
         }
         double x_dist
-            = std::sqrt( std::pow( wm.self().playerType().kickableArea(), 2.0 )
-                         - std::pow( y_dist, 2.0 ) )
+            = std::sqrt( std::pow( wm.self().playerType().kickableArea(), 2 )
+                         - std::pow( y_dist, 2 ) )
             - 0.2 - std::min( 0.6, my_move_dist * 0.05 );
-        control_dist = std::sqrt( std::pow( x_dist, 2.0 )
-                                  + std::pow( y_dist, 2.0 ) );
+        control_dist = std::sqrt( std::pow( x_dist, 2 )
+                                  + std::pow( y_dist, 2 ) );
         add_angle_abs = std::fabs( AngleDeg::atan2_deg( y_dist, x_dist ) );
     }
 
@@ -1018,7 +1017,7 @@ Body_Dribble2007::doKickDashes( PlayerAgent * agent,
         = std::pow( wm.self().playerType().playerSize()
                     + ServerParam::i().ballSize()
                     + 0.15,
-                    2.0);
+                    2 );
     if ( ( wm.ball().rpos()
            + required_first_vel - wm.self().vel() ).r2() // next rel pos
          < collide_dist2 )
