@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -52,6 +52,7 @@
 #include <rcsc/param/param_map.h>
 #include <rcsc/param/conf_file_parser.h>
 #include <rcsc/param/cmd_line_parser.h>
+#include <rcsc/version.h>
 
 #include <sstream>
 #include <cstring>
@@ -262,9 +263,9 @@ CoachAgent::initImpl( CmdLineParser & cmd_parser )
     cmd_parser.parse( system_param_map );
     if ( help )
     {
+        std::cout << copyright() << std::endl;
         system_param_map.printHelp( std::cout );
         coach_param_map.printHelp( std::cout );
-        M_client->setServerAlive( false );
         return false;
     }
 
@@ -278,12 +279,13 @@ CoachAgent::initImpl( CmdLineParser & cmd_parser )
     // analyze command line for coach options
     cmd_parser.parse( coach_param_map );
 
-    //if ( ! cmd_parser.invalidOptions().empty() )
-    //{
-    //    std::cerr << "coach: ***WARNING*** detected invalid options: ";
-    //    cmd_parser.printError( std::cerr );
-    //    std::cerr << std::endl;
-    //}
+    if ( config().version() < 1.0
+         || 14.0 <= config().version() )
+    {
+        std::cerr << "Unsupported client version: " << config().version()
+                  << std::endl;
+        return false;
+    }
 
     if ( config().debug() )
     {
@@ -300,6 +302,13 @@ CoachAgent::initImpl( CmdLineParser & cmd_parser )
         dlog.setLogFlag( &world().time(), Logger::CLEAR, config().debugClear() );
         dlog.setLogFlag( &world().time(), Logger::TEAM, config().debugTeam() );
         dlog.setLogFlag( &world().time(), Logger::ROLE, config().debugRole() );
+        dlog.setLogFlag( &world().time(), Logger::ANALYZER, config().debugAnalyzer() );
+        dlog.setLogFlag( &world().time(), Logger::RECURSIVE, config().debugRecursive() );
+    }
+
+    if ( config().offlineClientMode() )
+    {
+        M_client->setClientMode( BasicClient::OFFLINE );
     }
 
     return true;
@@ -337,6 +346,14 @@ CoachAgent::handleStart()
         M_client->setServerAlive( false );
     }
 
+    if ( config().offlineLogging() )
+    {
+        if ( ! openOfflineLog() )
+        {
+            return false;
+        }
+    }
+
     sendInitCommand();
     return true;
 }
@@ -344,6 +361,27 @@ CoachAgent::handleStart()
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+bool
+CoachAgent::handleStartOffline()
+{
+    if ( ! M_client )
+    {
+        return false;
+    }
+
+    if ( ! openOfflineLog() )
+    {
+        return false;
+    }
+
+    M_client->setServerAlive( true );
+    return true;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 void
 CoachAgent::handleMessage()
@@ -396,6 +434,38 @@ CoachAgent::handleMessage()
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+void
+CoachAgent::handleMessageOffline()
+{
+    if ( ! M_client )
+    {
+        std::cerr << "CoachAgent::handleMessageOffline(). Client is not registered."
+                  << std::endl;
+        return;
+    }
+
+    if ( M_client->recvMessage() > 0 )
+    {
+        parse( M_client->message() );
+    }
+
+    if ( M_impl->think_received_ )
+    {
+        dlog.addText( Logger::SYSTEM,
+                      __FILE__": Got think message: decide action" );
+#if 0
+        std::cout << config().teamName() << " coach: "
+                  << world().time()
+                  << " action" << std::endl;
+#endif
+        action();
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 void
 CoachAgent::handleTimeout( const int /*timeout_count*/,
@@ -494,6 +564,74 @@ CoachAgent::finalize()
               << std::endl;
 }
 
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+bool
+CoachAgent::openOfflineLog()
+{
+    std::string filepath = config().logDir();
+
+    if ( ! filepath.empty() )
+    {
+        if ( *filepath.rbegin() != '/' )
+        {
+            filepath += '/';
+        }
+    }
+
+    filepath += config().teamName();
+    filepath += "-coach";
+    filepath += config().offlineLogExt();
+
+    if ( ! M_client->openOfflineLog( filepath ) )
+    {
+        std::cerr << "Failed to open the offline client log file ["
+                  << filepath
+                  << "]" << std::endl;
+        M_client->setServerAlive( false );
+        return false;
+    }
+
+    return true;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+bool
+CoachAgent::openDebugLog()
+{
+    std::string filepath = config().logDir();
+
+    if ( ! filepath.empty() )
+    {
+        if ( *filepath.rbegin() != '/' )
+        {
+            filepath += '/';
+        }
+    }
+
+    filepath += config().teamName();
+    filepath += "-coach";
+    filepath += config().debugLogExt();
+
+    dlog.open( filepath );
+
+    if ( ! dlog.isOpen() )
+    {
+        std::cerr << config().teamName() << " coach: "
+                  << " Failed to open the debug log file [" << filepath << "]"
+                  << std::endl;
+        M_client->setServerAlive( false );
+        return false;
+    }
+
+    return true;
+}
+
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -501,6 +639,12 @@ CoachAgent::finalize()
 void
 CoachAgent::action()
 {
+    if ( config().offlineLogging()
+         && ! ServerParam::i().synchMode() )
+    {
+        M_client->printOfflineThink();
+    }
+
     if ( M_impl->last_decision_time_ != M_impl->current_time_ )
     {
         actionImpl();
@@ -626,33 +770,11 @@ CoachAgent::analyzeInit( const char * msg )
     SideID side_id = ( side == 'l'
                        ? LEFT
                        : RIGHT );
-    M_worldmodel.init( side_id );
+    M_worldmodel.init( side_id, config().version() );
 
     if ( config().debug() )
     {
-        std::ostringstream ostrm;
-        std::string file_dir = config().logDir();
-        if ( file_dir.empty() )
-        {
-            ostrm << "./";
-        }
-        else
-        {
-            ostrm << file_dir;
-            if ( file_dir[file_dir.length() - 1] != '/' )
-            {
-                ostrm << '/';
-            }
-        }
-        ostrm << config().teamName() << "-coach"
-              << config().logExt();
-        dlog.open( ostrm.str().c_str() );
-        if ( ! dlog.isOpen() )
-        {
-            std::cerr << config().teamName() << " coach: "
-                      << " Failed to open file [" << ostrm.str() << "]"
-                      << std::endl;
-        }
+        openDebugLog();
     }
 
     // send specific settings
