@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -154,6 +154,17 @@ protected:
     virtual
     bool handleStart();
 
+    /*!
+      \brief handle start event in offline client mode.
+      \return status of start procedure.
+
+      This method is called on the top of BasicClient::run() method.
+      The concrete agent must connect to the server and send init command.
+      Do NOT call this method by yourself!
+    */
+    virtual
+    bool handleStartOffline();
+
     /*!
       \brief handle server message event
 
@@ -162,6 +173,15 @@ protected:
     virtual
     void handleMessage();
 
+
+    /*!
+      \brief handle offline client log messages in offline client mode.
+
+      Do NOT call this method by yourself!
+     */
+    virtual
+    void handleMessageOffline();
+
     /*!
       \brief handle timeout event
       \param timeout_count count of timeout without sensory message.
@@ -200,6 +220,16 @@ public:
 
 private:
 
+    /*!
+      \brief open offline client log file.
+    */
+    bool openOfflineLog();
+
+    /*!
+      \brief open debug log file.
+    */
+    bool openDebugLog();
+
     /*!
       \brief send init or reconnect command to server
 
