@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -36,17 +36,25 @@
 #include <rcsc/rcg/types.h>
 #include <rcsc/game_time.h>
 
+#include <string>
 #include <iostream>
 
 namespace rcsc {
 
+/*!
+  \class GlobalVisualSensor
+  \brief visual sensor for coach/trainer
+ */
 class GlobalVisualSensor {
 private:
     //! last updated time
     GameTime M_time;
 
+    std::string M_team_name_left; //!< analyzed left team name string
+    std::string M_team_name_right; //!< analyzed right team name string
+
     GlobalBallObject M_ball; //!< analyzed ball info
-    GlobalPlayerVector M_players; //!< analyzed all players
+    std::vector< GlobalPlayerObject > M_players; //!< analyzed all players
 
 public:
     /*!
@@ -69,6 +77,26 @@ public:
           return M_time;
       }
 
+    /*!
+      \brief get an analyzed left team name
+      \return const reference to the string variable
+     */
+    const
+    std::string & teamNameLeft() const
+      {
+          return M_team_name_left;
+      }
+
+    /*!
+      \brief get an analyzed right team name
+      \return const reference to the string variable
+     */
+    const
+    std::string & teamNameRight() const
+      {
+          return M_team_name_right;
+      }
+
     /*!
       \brief get analyzed ball info
       \return const reference to the analyzed data
@@ -84,7 +112,7 @@ public:
       \return const reference to the player container
     */
     const
-    GlobalPlayerVector & players() const
+    std::vector< GlobalPlayerObject > & players() const
       {
           return M_players;
       }
