@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -49,15 +49,16 @@ namespace rcsc {
 
 */
 GlobalWorldModel::GlobalWorldModel()
-    : M_time( -1, 0 )
-    , M_see_time( -1, 0 )
-    , M_our_side( NEUTRAL )
-    , M_player_type_analyzer( *this )
-    , M_change_count_left( 0 )
-    , M_change_count_right( 0 )
-    , M_last_playon_start( 0 )
-    , M_freeform_allowed_count( ServerParam::i().coachSayCountMax() )
-    , M_freeform_send_count( 0 )
+    : M_client_version( 0 ),
+      M_time( -1, 0 ),
+      M_see_time( -1, 0 ),
+      M_our_side( NEUTRAL ),
+      M_player_type_analyzer( *this ),
+      M_change_count_left( 0 ),
+      M_change_count_right( 0 ),
+      M_last_playon_start( 0 ),
+      M_freeform_allowed_count( ServerParam::i().coachSayCountMax() ),
+      M_freeform_send_count( 0 )
 {
     for ( int i = 0; i < 11; i++ )
     {
@@ -74,9 +75,11 @@ GlobalWorldModel::GlobalWorldModel()
 
 */
 void
-GlobalWorldModel::init( const SideID side )
+GlobalWorldModel::init( const SideID side,
+                        const int client_version )
 {
     M_our_side = side;
+    M_client_version = client_version;
 }
 
 /*-------------------------------------------------------------------*/
@@ -443,25 +446,25 @@ GlobalWorldModel::getBallStatus() const
     static const double WIDTH
         = ServerParam::i().goalHalfWidth()
         + ServerParam::DEFAULT_GOAL_POST_RADIUS;
-    static const Rect2D GOAL_L( ( - ServerParam::DEFAULT_PITCH_LENGTH * 0.5
-                                  - ServerParam::DEFAULT_GOAL_DEPTH
-                                  - ServerParam::i().ballSize() ),
-                                - WIDTH * 0.5,
-                                ServerParam::DEFAULT_GOAL_DEPTH,
-                                WIDTH );
-    static const Rect2D GOAL_R( ( ServerParam::DEFAULT_PITCH_LENGTH * 0.5
-                                  + ServerParam::i().ballSize() ),
-                                - WIDTH * 0.5,
-                                ServerParam::DEFAULT_GOAL_DEPTH,
-                                WIDTH );
-    static const Rect2D PITCH( ( - ServerParam::DEFAULT_PITCH_LENGTH * 0.5
-                                 - ServerParam::i().ballSize() * 0.5 ),
-                               ( - ServerParam::DEFAULT_PITCH_WIDTH * 0.5
-                                 - ServerParam::i().ballSize() * 0.5 ),
-                               ( ServerParam::DEFAULT_PITCH_LENGTH
-                                 + ServerParam::i().ballSize() ),
-                               ( ServerParam::DEFAULT_PITCH_WIDTH
-                                 + ServerParam::i().ballSize() ) );
+    static const Rect2D GOAL_L( Vector2D( - ServerParam::DEFAULT_PITCH_LENGTH * 0.5
+                                          - ServerParam::DEFAULT_GOAL_DEPTH
+                                          - ServerParam::i().ballSize(),
+                                          - WIDTH * 0.5 ),
+                                Size2D( ServerParam::DEFAULT_GOAL_DEPTH,
+                                        WIDTH ) );
+    static const Rect2D GOAL_R( Vector2D( ServerParam::DEFAULT_PITCH_LENGTH * 0.5
+                                          + ServerParam::i().ballSize(),
+                                          - WIDTH * 0.5 ),
+                                Size2D( ServerParam::DEFAULT_GOAL_DEPTH,
+                                        WIDTH ) );
+    static const Rect2D PITCH( Vector2D( - ServerParam::DEFAULT_PITCH_LENGTH * 0.5
+                                         - ServerParam::i().ballSize() * 0.5,
+                                         - ServerParam::DEFAULT_PITCH_WIDTH * 0.5
+                                         - ServerParam::i().ballSize() * 0.5 ),
+                               Size2D( ServerParam::DEFAULT_PITCH_LENGTH
+                                       + ServerParam::i().ballSize(),
+                                       ServerParam::DEFAULT_PITCH_WIDTH
+                                       + ServerParam::i().ballSize() ) );
 
 
     if ( GOAL_L.contains( M_ball.pos() ) )
@@ -485,6 +488,54 @@ GlobalWorldModel::getBallStatus() const
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+const
+GlobalPlayerObject *
+GlobalWorldModel::teammate( const int unum ) const
+{
+    const std::vector< const GlobalPlayerObject * > & cont = teammates();
+
+    const std::vector< const GlobalPlayerObject * >::const_iterator end = cont.end();
+    for ( std::vector< const GlobalPlayerObject * >::const_iterator p = cont.begin();
+          p != end;
+          ++p )
+    {
+        if ( (*p)->unum() == unum )
+        {
+            return *p;
+        }
+    }
+
+    return static_cast< const GlobalPlayerObject * >( 0 );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+const
+GlobalPlayerObject *
+GlobalWorldModel::opponent( const int unum ) const
+{
+    const std::vector< const GlobalPlayerObject * > & cont = opponents();
+
+    const std::vector< const GlobalPlayerObject * >::const_iterator end = cont.end();
+    for ( std::vector< const GlobalPlayerObject * >::const_iterator p = cont.begin();
+          p != end;
+          ++p )
+    {
+        if ( (*p)->unum() == unum )
+        {
+            return *p;
+        }
+    }
+
+    return static_cast< const GlobalPlayerObject * >( 0 );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 bool
 GlobalWorldModel::canSendFreeform() const
@@ -551,7 +602,7 @@ GlobalPlayerObject *
 GlobalWorldModel::getPlayerNearestTo( const Vector2D & point ) const
 {
     const GlobalPlayerObject * ptr = static_cast< GlobalPlayerObject * >( 0 );
-    double max_dist2 = 20000.0;
+    double max_dist2 = 200000.0;
 
     const std::list< GlobalPlayerObject >::const_iterator end = M_players.end();
     for ( std::list< GlobalPlayerObject >::const_iterator it = M_players.begin();
