@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -73,6 +73,8 @@ AudioMemory::setBall( const int sender,
 
     M_ball.push_back(  Ball( sender, pos, vel ) );
     M_ball_time = current;
+
+    M_time = current;
 }
 
 
@@ -99,6 +101,8 @@ AudioMemory::setPass( const int sender,
 
     M_pass.push_back( Pass( sender, receiver, pos ) );;
     M_pass_time = current;
+
+    M_time = current;
 }
 
 /*-------------------------------------------------------------------*/
@@ -149,6 +153,8 @@ AudioMemory::setIntercept( const int sender,
                                                  std::max( 0, cycle - 1 ) ) );
         M_opp_intercept_time = current;
     }
+
+    M_time = current;
 }
 
 /*-------------------------------------------------------------------*/
@@ -174,6 +180,8 @@ AudioMemory::setOpponentGoalie( const int sender,
 
     M_goalie.push_back( Goalie( sender, pos, body ) );
     M_goalie_time = current;
+
+    M_time = current;
 }
 
 /*-------------------------------------------------------------------*/
@@ -184,7 +192,33 @@ void
 AudioMemory::setPlayer( const int sender,
                         const int unum,
                         const Vector2D & pos,
-                        const AngleDeg & body,
+                        const GameTime & current )
+{
+    dlog.addText( Logger::WORLD,
+                  __FILE__": set heard player. sender=%d "
+                  "unum=%d pos=(%.2f, %.2f) no body",
+                  sender,
+                  unum, pos.x, pos.y );
+    if ( M_player_time != current )
+    {
+        M_player.clear();
+    }
+
+    M_player.push_back( Player( sender, unum, pos ) );
+    M_player_time = current;
+
+    M_time = current;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+AudioMemory::setPlayer( const int sender,
+                        const int unum,
+                        const Vector2D & pos,
+                        const double & body,
                         const double & stamina,
                         const GameTime & current )
 {
@@ -192,7 +226,7 @@ AudioMemory::setPlayer( const int sender,
                   __FILE__": set heard player. sender=%d "
                   "unum=%d pos=(%.2f, %.2f) body=%.1f",
                   sender,
-                  unum, pos.x, pos.y, body.degree() );
+                  unum, pos.x, pos.y, body );
     if ( M_player_time != current )
     {
         M_player.clear();
@@ -200,6 +234,8 @@ AudioMemory::setPlayer( const int sender,
 
     M_player.push_back( Player( sender, unum, pos, body, stamina ) );
     M_player_time = current;
+
+    M_time = current;
 }
 
 /*-------------------------------------------------------------------*/
@@ -222,6 +258,8 @@ AudioMemory::setOffsideLine( const int sender,
 
     M_offside_line.push_back( OffsideLine( sender, offside_line_x ) );
     M_offside_line_time = current;
+
+    M_time = current;
 }
 
 /*-------------------------------------------------------------------*/
@@ -244,6 +282,8 @@ AudioMemory::setDefenseLine( const int sender,
 
     M_defense_line.push_back( DefenseLine( sender, defense_line_x ) );
     M_defense_line_time = current;
+
+    M_time = current;
 }
 
 /*-------------------------------------------------------------------*/
@@ -265,6 +305,8 @@ AudioMemory::setWaitRequest( const int sender,
 
     M_wait_request.push_back( WaitRequest( sender ) );
     M_wait_request_time = current;
+
+    M_time = current;
 }
 
 /*-------------------------------------------------------------------*/
@@ -287,6 +329,34 @@ AudioMemory::setPassRequest( const int sender,
 
     M_pass_request.push_back( PassRequest( sender, request_pos ) );
     M_pass_request_time = current;
+
+    M_time = current;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+AudioMemory::setRunRequest( const int sender,
+                            const int runner,
+                            const Vector2D & request_pos,
+                            const GameTime & current )
+{
+    dlog.addText( Logger::WORLD,
+                  __FILE__": set heard run request."
+                  " sender=%d runner=%d pos=(%.1f %.1f)",
+                  sender, runner, request_pos.x, request_pos.y );
+
+    if ( M_run_request_time != current )
+    {
+        M_run_request.clear();
+    }
+
+    M_run_request.push_back( RunRequest( sender, runner, request_pos ) );
+    M_run_request_time = current;
+
+    M_time = current;
 }
 
 /*-------------------------------------------------------------------*/
@@ -309,6 +379,8 @@ AudioMemory::setStamina( const int sender,
 
     M_stamina.push_back( Stamina( sender, rate ) );
     M_stamina_time = current;
+
+    M_time = current;
 }
 
 /*-------------------------------------------------------------------*/
@@ -331,6 +403,8 @@ AudioMemory::setRecovery( const int sender,
 
     M_recovery.push_back( Recovery( sender, rate ) );
     M_recovery_time = current;
+
+    M_time = current;
 }
 
 /*-------------------------------------------------------------------*/
@@ -357,6 +431,8 @@ AudioMemory::setDribbleTarget( const int sender,
 
     M_dribble.push_back( Dribble( sender, pos, queue_count ) );
     M_dribble_time = current;
+
+    M_time = current;
 }
 
 /*-------------------------------------------------------------------*/
@@ -383,6 +459,181 @@ AudioMemory::setFreeMessage( const int sender,
 
     M_free_message.push_back( FreeMessage( sender, msg ) );
     M_free_message_time = current;
+
+    M_time = current;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+std::ostream &
+AudioMemory::printDebug( std::ostream & os ) const
+{
+    //
+    // TODO: reimplemented using virtual method.
+    //
+
+    if ( time() == ballTime() )
+    {
+        for ( std::vector< Ball >::const_iterator it = ball().begin();
+              it != ball().end();
+              ++it )
+        {
+            os << '(' << it->sender_ << " \"Ball\")";
+        }
+    }
+
+    if ( time() == passTime() )
+    {
+        for ( std::vector< Pass >::const_iterator it = pass().begin();
+              it != pass().end();
+              ++it )
+        {
+            os << '(' << it->sender_ << " \"Pass\")";
+        }
+    }
+
+    if ( time() == ourInterceptTime() )
+    {
+        for ( std::vector< OurIntercept >::const_iterator it = ourIntercept().begin();
+              it != ourIntercept().end();
+              ++it )
+        {
+            os << '(' << it->sender_ << " \"OurIntercept:" << it->interceptor_ << "\")";
+        }
+    }
+
+    if ( time() == oppInterceptTime() )
+    {
+        for ( std::vector< OppIntercept >::const_iterator it = oppIntercept().begin();
+              it != oppIntercept().end();
+              ++it )
+        {
+            os << '(' << it->sender_ << " \"OppIntercept:" << it->interceptor_ << "\")";
+        }
+    }
+
+    if ( time() == goalieTime() )
+    {
+        for ( std::vector< Goalie >::const_iterator it = goalie().begin();
+              it != goalie().end();
+              ++it )
+        {
+            os << '(' << it->sender_ << " \"Goalie\")";
+        }
+    }
+
+    if ( time() == playerTime() )
+    {
+        for ( std::vector< Player >::const_iterator it = player().begin();
+              it != player().end();
+              ++it )
+        {
+            os << '(' << it->sender_;
+            if ( it->unum_ <= 11 )
+            {
+                os << " \"T_" << it->unum_;
+            }
+            else
+            {
+                os << " \"O_" << it->unum_ - 11;
+            }
+            os << "\")";
+        }
+    }
+
+    if ( time() == offsideLineTime() )
+    {
+        for ( std::vector< OffsideLine >::const_iterator it = offsideLine().begin();
+              it != offsideLine().end();
+              ++it )
+        {
+            os << '(' << it->sender_ << " \"OffsideLine\")";
+        }
+    }
+
+    if ( time() == defenseLineTime() )
+    {
+        for ( std::vector< DefenseLine >::const_iterator it = defenseLine().begin();
+              it != defenseLine().end();
+              ++it )
+        {
+            os << '(' << it->sender_ << " \"DefenseLine\")";
+        }
+    }
+
+    if ( time() == waitRequestTime() )
+    {
+        for ( std::vector< WaitRequest >::const_iterator it = waitRequest().begin();
+              it != waitRequest().end();
+              ++it )
+        {
+            os << '(' << it->sender_ << " \"Wait\")";
+        }
+    }
+
+    if ( time() == passRequestTime() )
+    {
+        for ( std::vector< PassRequest >::const_iterator it = passRequest().begin();
+              it != passRequest().end();
+              ++it )
+        {
+            os << '(' << it->sender_ << " \"PassRequest\")";
+        }
+    }
+
+    if ( time() == runRequestTime() )
+    {
+        for ( std::vector< RunRequest >::const_iterator it = runRequest().begin();
+              it != runRequest().end();
+              ++it )
+        {
+            os << '(' << it->sender_ << " \"RunRequest\")";
+        }
+    }
+
+    if ( time() == staminaTime() )
+    {
+        for ( std::vector< Stamina >::const_iterator it = stamina().begin();
+              it != stamina().end();
+              ++it )
+        {
+            os << '(' << it->sender_ << " \"Stamina\")";
+        }
+    }
+
+    if ( time() == recoveryTime() )
+    {
+        for ( std::vector< Recovery >::const_iterator it = recovery().begin();
+              it != recovery().end();
+              ++it )
+        {
+            os << '(' << it->sender_ << " \"Recovery\")";
+        }
+    }
+
+    if ( time() == dribbleTime() )
+    {
+        for ( std::vector< Dribble >::const_iterator it = dribble().begin();
+              it != dribble().end();
+              ++it )
+        {
+            os << '(' << it->sender_ << " \"Dribble\")";
+        }
+    }
+
+    if ( time() == freeMessageTime() )
+    {
+        for ( std::vector< FreeMessage >::const_iterator it = freeMessage().begin();
+              it != freeMessage().end();
+              ++it )
+        {
+            os << '(' << it->sender_ << " \"FreeMessage\")";
+        }
+    }
+
+    return os;
 }
 
 }
