@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,6 +35,7 @@
 #include <rcsc/geom/vector_2d.h>
 #include <rcsc/game_time.h>
 
+#include <iostream>
 #include <string>
 #include <vector>
 
@@ -158,9 +159,9 @@ public:
      */
     struct Player {
         int sender_; //!< player info sender number
-        int unum_; //!< heard player unum
+        int unum_; //!< heard player unum. if opponent player, += 11
         Vector2D pos_; //!< heard player position
-        AngleDeg body_; //!< heard player's body angle
+        double body_; //!< heard player's body angle
         double stamina_; //!< heard player's stamina info. negative value means stamina is unknown
 
         /*!
@@ -169,7 +170,7 @@ public:
         Player( const int sender,
                 const int unum,
                 const Vector2D & pos,
-                const AngleDeg & body,
+                const double & body = -360.0,
                 const double & stamina = -1.0 )
             : sender_( sender )
             , unum_( unum )
@@ -248,6 +249,27 @@ public:
           { }
     };
 
+    /*!
+      \struct RunRequest
+      \brief run request info
+     */
+    struct RunRequest {
+        int sender_; //!< run request sendeer number
+        int runner_; //!< heard runner number
+        Vector2D pos_; //!< heard run request position
+
+        /*!
+          \brief initialize all member
+        */
+        RunRequest( const int sender,
+                    const int runner,
+                    const Vector2D & pos )
+            : sender_( sender )
+            , runner_( runner )
+            , pos_( pos )
+          { }
+    };
+
     /*!
       \struct Stamina
       \brief stamina info
@@ -325,6 +347,9 @@ public:
 
 protected:
 
+    //! last updated time
+    GameTime M_time;
+
     std::vector< Ball > M_ball; //!< heard info
     GameTime M_ball_time; //!< info heard time
 
@@ -355,6 +380,9 @@ protected:
     std::vector< PassRequest > M_pass_request; //!< heard info
     GameTime M_pass_request_time; //!< info heard time
 
+    std::vector< RunRequest > M_run_request; //!< heard info
+    GameTime M_run_request_time; //!< info heard time
+
     std::vector< Stamina > M_stamina; //!< heard info
     GameTime M_stamina_time; //!< info heard time
 
@@ -386,9 +414,14 @@ public:
     ~AudioMemory()
       { }
 
-
     // accessor methods
 
+    const
+    GameTime & time() const
+      {
+          return M_time;
+      }
+
     /*!
       \brief get heard ball info
       \return ball info container
@@ -589,6 +622,26 @@ public:
           return M_pass_request_time;
       }
 
+    /*!
+      \brief get run request info
+      \return run request info container
+    */
+    const
+    std::vector< RunRequest > & runRequest() const
+      {
+          return M_run_request;
+      }
+
+    /*!
+      \brief get run request heard time
+      \return time value
+    */
+    const
+    GameTime & runRequestTime() const
+      {
+          return M_run_request_time;
+      }
+
     /*!
       \brief get heard stamina info
       \return stamina info container
@@ -724,6 +777,19 @@ public:
                             const AngleDeg & body,
                             const GameTime & current );
 
+    /*!
+      \brief set heard player info
+      \param sender sender's uniform number
+      \param unum heard player unum
+      \param pos heard player pos
+      \param current current game time
+    */
+    virtual
+    void setPlayer( const int sender,
+                    const int unum,
+                    const Vector2D & pos,
+                    const GameTime & current );
+
     /*!
       \brief set heard player info
       \param sender sender's uniform number
@@ -737,7 +803,7 @@ public:
     void setPlayer( const int sender,
                     const int unum,
                     const Vector2D & pos,
-                    const AngleDeg & body,
+                    const double & body,
                     const double & stamina,
                     const GameTime & current );
 
@@ -783,6 +849,19 @@ public:
                          const Vector2D & request_pos,
                          const GameTime & current );
 
+    /*!
+      \brief set heard run request info
+      \param sender message sender's uniform number
+      \param runner runner's uniform number
+      \param request_pos request position
+      \param current current game time
+    */
+    virtual
+    void setRunRequest( const int sender,
+                        const int runner,
+                        const Vector2D & request_pos,
+                        const GameTime & current );
+
     /*!
       \brief set heard stamina info
       \param sender message sender's uniform number
@@ -829,6 +908,8 @@ public:
                          const std::string & msg,
                          const GameTime & current );
 
+    virtual
+    std::ostream & printDebug( std::ostream & os ) const;
 };
 
 }
