@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -55,11 +55,14 @@ namespace rcsc {
 
 */
 BasicClient::BasicClient()
-    : M_server_alive( false )
-    , M_interval_msec( 10 )
-    , M_compression_level( 0 )
+    : M_client_mode( ONLINE ),
+      M_server_alive( false ),
+      M_interval_msec( 10 ),
+      M_compression_level( 0 )
 {
     std::memset( M_message, 0, MAX_MESG );
+    M_compression_message.reserve( MAX_MESG );
+    M_decompression_message.reserve( MAX_MESG );
 }
 
 /*-------------------------------------------------------------------*/
@@ -69,6 +72,11 @@ BasicClient::BasicClient()
 BasicClient::~BasicClient()
 {
     //std::cerr << "delete BasicClient" << std::endl;
+    if ( M_offline_out.is_open() )
+    {
+        M_offline_out.flush();
+        M_offline_out.close();
+    }
 }
 
 /*-------------------------------------------------------------------*/
@@ -80,6 +88,25 @@ BasicClient::run( SoccerAgent * agent )
 {
     assert( agent );
 
+    if ( clientMode() == ONLINE )
+    {
+        runOnline( agent );
+    }
+    else // if ( clientMode() == OFFLINE )
+    {
+        runOffline( agent );
+    }
+
+    agent->handleExit();
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+BasicClient::runOnline( SoccerAgent * agent )
+{
     if ( ! agent->handleStart() )
     {
         agent->handleExit();
@@ -130,8 +157,25 @@ BasicClient::run( SoccerAgent * agent )
             agent->handleMessage();
         }
     }
+}
 
-    agent->handleExit();
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+BasicClient::runOffline( SoccerAgent * agent )
+{
+    if ( ! agent->handleStartOffline() )
+    {
+        agent->handleExit();
+        return;
+    }
+
+    while ( isServerAlive() )
+    {
+        agent->handleMessageOffline();
+    }
 }
 
 /*-------------------------------------------------------------------*/
@@ -211,14 +255,66 @@ BasicClient::setCompressionLevel( const int level )
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+bool
+BasicClient::openOfflineLog( const std::string & filepath )
+{
+    if ( clientMode() == ONLINE )
+    {
+        M_offline_out.close();
+        M_offline_out.open( filepath.c_str() );
+        if ( ! M_offline_out.is_open() )
+        {
+            return false;
+        }
+
+        if ( ! M_decompression_message.empty() )
+        {
+            M_offline_out << M_decompression_message << std::endl;
+        }
+
+        return true;
+    }
+    else if ( clientMode() == OFFLINE )
+    {
+        M_offline_in.close();
+        M_offline_in.open( filepath.c_str() );
+
+        return M_offline_in.is_open();
+    }
+
+    return false;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+BasicClient::printOfflineThink()
+{
+    if ( M_offline_out.is_open() )
+    {
+        M_offline_out << "(think)" << std::endl;
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 bool
 BasicClient::connectTo( const char * hostname,
                         const int port,
                         const long & interval_msec )
 {
-    M_socket
-        = boost::shared_ptr< UDPSocket >( new UDPSocket( hostname, port ) );
+    if ( clientMode() != ONLINE )
+    {
+        M_socket.reset();
+        return true;
+    }
+
+    M_socket = boost::shared_ptr< UDPSocket >( new UDPSocket( hostname, port ) );
 
     if ( ! M_socket
          || M_socket->fd() == -1 )
@@ -243,6 +339,11 @@ BasicClient::connectTo( const char * hostname,
 int
 BasicClient::sendMessage( const char * msg )
 {
+    if ( clientMode() != ONLINE )
+    {
+        return 1;
+    }
+
     if ( ! M_socket )
     {
         return 0;
@@ -279,6 +380,27 @@ BasicClient::sendMessage( const char * msg )
 int
 BasicClient::recvMessage()
 {
+    //
+    // offline mode
+    //
+
+    if ( clientMode() == OFFLINE )
+    {
+        while ( std::getline( M_offline_in, M_decompression_message ) )
+        {
+            if ( M_decompression_message.empty() ) continue;
+
+            return M_decompression_message.size();
+        }
+
+        setServerAlive( false );
+        return 0;
+    }
+
+    //
+    // online mode
+    //
+
     if ( ! M_socket )
     {
         return 0;
@@ -302,6 +424,11 @@ BasicClient::recvMessage()
             M_message[n] = '\0';
             M_decompression_message = M_message;
         }
+
+        if ( M_offline_out.is_open() )
+        {
+            M_offline_out << M_decompression_message << '\n';
+        }
     }
 
     return n;
