@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -183,60 +183,119 @@ private:
 
 public:
 
+    /*!
+      \brief get the player_type parameter
+      \return player_type parameter
+     */
     int id() const
       {
           return M_id;
       }
+
+    /*!
+      \brief get the player_type parameter
+      \return player_type parameter
+     */
     const
     double & playerSpeedMax() const
       {
           return M_player_speed_max;
       }
+
+    /*!
+      \brief get the player_type parameter
+      \return player_type parameter
+     */
     const
     double & staminaIncMax() const
       {
           return M_stamina_inc_max;
       }
+
+    /*!
+      \brief get the player_type parameter
+      \return player_type parameter
+     */
     const
     double & playerDecay() const
       {
           return M_player_decay;
       }
+
+    /*!
+      \brief get the player_type parameter
+      \return player_type parameter
+     */
     const
     double & inertiaMoment() const
       {
           return M_inertia_moment;
       }
+
+    /*!
+      \brief get the player_type parameter
+      \return player_type parameter
+     */
     const
     double & dashPowerRate() const
       {
           return M_dash_power_rate;
       }
+
+    /*!
+      \brief get the player_type parameter
+      \return player_type parameter
+     */
     const
     double & playerSize() const
       {
           return M_player_size;
       }
+
+    /*!
+      \brief get the player_type parameter
+      \return player_type parameter
+     */
     const
     double & kickableMargin() const
       {
           return M_kickable_margin;
       }
+
+    /*!
+      \brief get the player_type parameter
+      \return player_type parameter
+     */
     const
     double & kickRand() const
       {
           return M_kick_rand;
       }
+
+    /*!
+      \brief get the player_type parameter
+      \return player_type parameter
+     */
     const
     double & extraStamina() const
       {
           return M_extra_stamina;
       }
+
+    /*!
+      \brief get the player_type parameter
+      \return player_type parameter
+     */
     const
     double & effortMax() const
       {
           return M_effort_max;
       }
+
+    /*!
+      \brief get the player_type parameter
+      \return player_type parameter
+     */
     const
     double & effortMin() const
       {
@@ -246,21 +305,40 @@ public:
     ////////////////////////////////////////////////
     // additional parameters
 
+    /*!
+      \brief get the maximum kickable radius
+      \return maximum kickable radius
+     */
     const
     double & kickableArea() const
       {
           return M_kickable_area;
       }
+
+    /*!
+      \brief get the reachable speed max
+      \return reachable speed max
+     */
     const
     double & realSpeedMax() const
       {
           return M_real_speed_max;
       }
+
+    /*!
+      \brief get the squared player speed max
+      \return squared player speed max
+     */
     const
     double & playerSpeedMax2() const
       {
           return M_player_speed_max2;
       }
+
+    /*!
+      \brief get the squared real speed max
+      \return squared real speed max
+     */
     const
     double & realSpeedMax2() const
       {
@@ -376,7 +454,7 @@ public:
                                     const double & effort ) const
       {
           return speed * ( ( 1.0 - playerDecay() )
-                           / (dashPowerRate() * effort ) );
+                           / ( dashPowerRate() * effort ) );
       }
 
     /*!
@@ -412,6 +490,15 @@ public:
           return effort * dashPowerRate();
       }
 
+    /*!
+      \brief calculate dash rate
+      \param effort current effort value
+      \param rel_dir dash direction
+      \return dash power rate multiplied by effort
+     */
+    double dashRate( const double & effort,
+                     const double & rel_dir ) const;
+
     /*!
       \brief calculate effective turn angle
       \param command_moment turn command argument
@@ -455,10 +542,10 @@ public:
       \param initial_pos initial point
       \param initial_vel initial velocity vector
       \param n_step cycles to be estimated
-      \rturn the reached point
+      \return the reached point
      */
-    Vector2D inertiaPoint( const Vector2D& initial_pos,
-                           const Vector2D& initial_vel,
+    Vector2D inertiaPoint( const Vector2D & initial_pos,
+                           const Vector2D & initial_vel,
                            const int n_step ) const
       {
           return inertia_n_step_point( initial_pos,
@@ -532,11 +619,11 @@ public:
       \param effort pointer to effort variable
       \param recovery pointer to recovery variable
     */
-    void predictStaminaAfterDash( const ServerParam & sparam,
-                                  const double & dash_power,
-                                  double * stamina,
-                                  double * effort,
-                                  double * recovery ) const;
+    void predictStaminaAfterOneDash( const ServerParam & sparam,
+                                     const double & dash_power,
+                                     double * stamina,
+                                     double * effort,
+                                     double * recovery ) const;
 
     /*!
       \brief predict stamina related values after nr dashes
@@ -547,7 +634,7 @@ public:
       \param effort pointer to effort variable
       \param recovery pointer to recovery variable
      */
-    void predictStaminaAfterNrDash( const ServerParam & sparam,
+    void predictStaminaAfterDashes( const ServerParam & sparam,
                                     const double & dash_power,
                                     const int n_dash,
                                     double * stamina,
