@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -315,7 +315,7 @@ public:
   \brief goalie info message parser
 
   format:
-  "g<unum_pos:4>"
+  "g<pos_body:4>"
   the length of message == 5
  */
 class GoalieMessageParser
@@ -954,8 +954,8 @@ public:
   \brief player info message parser
 
   format:
-  "P<unum_pos_body:>"
-  the length of message == 5
+  "P<unum_pos:3>"
+  the length of message == 4
  */
 class OnePlayerMessageParser
     : public SayMessageParser {
@@ -999,7 +999,149 @@ public:
     static
     std::size_t slength()
       {
-          return 5;
+          return 4;
+      }
+
+    /*!
+      \brief virtual method which analyzes audio messages.
+      \param sender sender's uniform number
+      \param dir sender's direction
+      \param msg raw audio message
+      \param current current game time
+      \retval bytes read if success
+      \retval 0 message ID is not match. other parser should be tried.
+      \retval -1 failed to parse
+    */
+    int parse( const int sender,
+               const double & dir,
+               const char * msg,
+               const GameTime & current );
+
+};
+
+/*-------------------------------------------------------------------*/
+/*!
+  \class TwoPlayerMessageParser
+  \brief 2 players info message parser
+
+  format:
+  "Q<unum_pos:3,unum_pos:3>"
+  the length of message == 7
+ */
+class TwoPlayerMessageParser
+    : public SayMessageParser {
+private:
+
+    //! pointer to the audio memory
+    boost::shared_ptr< AudioMemory > M_memory;
+
+public:
+
+    /*!
+      \brief construct with audio memory
+      \param memory pointer to the memory
+     */
+    explicit
+    TwoPlayerMessageParser( boost::shared_ptr< AudioMemory > memory );
+
+    /*!
+      \brief get the header character.
+      \return header character.
+     */
+    static
+    char sheader()
+      {
+          return 'Q';
+      }
+
+    /*!
+      \brief get the header character.
+      \return header character.
+     */
+    char header() const
+      {
+          return sheader();
+      }
+
+    /*!
+      \brief get the length of this message.
+      \return the length of encoded message
+    */
+    static
+    std::size_t slength()
+      {
+          return 7;
+      }
+
+    /*!
+      \brief virtual method which analyzes audio messages.
+      \param sender sender's uniform number
+      \param dir sender's direction
+      \param msg raw audio message
+      \param current current game time
+      \retval bytes read if success
+      \retval 0 message ID is not match. other parser should be tried.
+      \retval -1 failed to parse
+    */
+    int parse( const int sender,
+               const double & dir,
+               const char * msg,
+               const GameTime & current );
+
+};
+
+/*-------------------------------------------------------------------*/
+/*!
+  \class ThreePlayerMessageParser
+  \brief 3 players info message parser
+
+  format:
+  "R<unum_pos:3,unum_pos:3,unum_pos:3>"
+  the length of message == 10
+ */
+class ThreePlayerMessageParser
+    : public SayMessageParser {
+private:
+
+    //! pointer to the audio memory
+    boost::shared_ptr< AudioMemory > M_memory;
+
+public:
+
+    /*!
+      \brief construct with audio memory
+      \param memory pointer to the memory
+     */
+    explicit
+    ThreePlayerMessageParser( boost::shared_ptr< AudioMemory > memory );
+
+    /*!
+      \brief get the header character.
+      \return header character.
+     */
+    static
+    char sheader()
+      {
+          return 'R';
+      }
+
+    /*!
+      \brief get the header character.
+      \return header character.
+     */
+    char header() const
+      {
+          return sheader();
+      }
+
+    /*!
+      \brief get the length of this message.
+      \return the length of encoded message
+    */
+    static
+    std::size_t slength()
+      {
+          return 10;
       }
 
     /*!
@@ -1090,14 +1232,156 @@ public:
 
 };
 
+/*-------------------------------------------------------------------*/
+/*!
+  \class TeammateMessageParser
+  \brief teammate info message parser
+
+  format:
+  "T<unum_pos_body>"
+  the length of message == 5
+ */
+class TeammateMessageParser
+    : public SayMessageParser {
+private:
+
+    //! pointer to the audio memory
+    boost::shared_ptr< AudioMemory > M_memory;
+
+public:
+
+    /*!
+      \brief construct with audio memory
+      \param memory pointer to the memory
+     */
+    explicit
+    TeammateMessageParser( boost::shared_ptr< AudioMemory > memory );
+
+    /*!
+      \brief get the header character.
+      \return header character.
+     */
+    static
+    char sheader()
+      {
+          return 'T';
+      }
+
+    /*!
+      \brief get the header character.
+      \return header character.
+     */
+    char header() const
+      {
+          return sheader();
+      }
+
+    /*!
+      \brief get the length of this message.
+      \return the length of encoded message
+    */
+    static
+    std::size_t slength()
+      {
+          return 5;
+      }
+
+    /*!
+      \brief virtual method which analyzes audio messages.
+      \param sender sender's uniform number
+      \param dir sender's direction
+      \param msg raw audio message
+      \param current current game time
+      \retval bytes read if success
+      \retval 0 message ID is not match. other parser should be tried.
+      \retval -1 failed to parse
+    */
+    int parse( const int sender,
+               const double & dir,
+               const char * msg,
+               const GameTime & current );
+
+};
+
+/*-------------------------------------------------------------------*/
+/*!
+  \class OpponentMessageParser
+  \brief opponent info message parser
+
+  format:
+  "O<unum_pos_body>"
+  the length of message == 5
+ */
+class OpponentMessageParser
+    : public SayMessageParser {
+private:
+
+    //! pointer to the audio memory
+    boost::shared_ptr< AudioMemory > M_memory;
+
+public:
+
+    /*!
+      \brief construct with audio memory
+      \param memory pointer to the memory
+     */
+    explicit
+    OpponentMessageParser( boost::shared_ptr< AudioMemory > memory );
+
+    /*!
+      \brief get the header character.
+      \return header character.
+     */
+    static
+    char sheader()
+      {
+          return 'O';
+      }
+
+    /*!
+      \brief get the header character.
+      \return header character.
+     */
+    char header() const
+      {
+          return sheader();
+      }
+
+    /*!
+      \brief get the length of this message.
+      \return the length of encoded message
+    */
+    static
+    std::size_t slength()
+      {
+          return 5;
+      }
+
+    /*!
+      \brief virtual method which analyzes audio messages.
+      \param sender sender's uniform number
+      \param dir sender's direction
+      \param msg raw audio message
+      \param current current game time
+      \retval bytes read if success
+      \retval 0 message ID is not match. other parser should be tried.
+      \retval -1 failed to parse
+    */
+    int parse( const int sender,
+               const double & dir,
+               const char * msg,
+               const GameTime & current );
+
+};
+
 /*-------------------------------------------------------------------*/
 /*!
   \class BallPlayerMessageParser
   \brief ball & player info message parser
 
   format:
-  "B<pos_vel_body:>"
-  the length of message == ??
+  "B<bpos_bvel_unum_ppos_pbody:9>"
+  the length of message == 10
  */
 class BallPlayerMessageParser
     : public SayMessageParser {
