@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -312,6 +312,18 @@ const double ServerParam::DEFAULT_MAX_TACKLE_POWER = 100.0;
 const double ServerParam::DEFAULT_MAX_BACK_TACKLE_POWER = 50.0;
 const double ServerParam::DEFAULT_PLAYER_SPEED_MAX_MIN = 0.8;
 const double ServerParam::DEFAULT_EXTRA_STAMINA = 0.0;
+const int ServerParam::DEFAULT_SYNCH_SEE_OFFSET = 30;
+
+const int ServerParam::EXTRA_HALF_TIME = 100;
+
+const double ServerParam::STAMINA_CAPACITY = -1.0; //148600.0;
+const double ServerParam::MAX_DASH_ANGLE = 0.0; //180.0;
+const double ServerParam::MIN_DASH_ANGLE = 0.0; //-180.0;
+const double ServerParam::DASH_ANGLE_STEP = 180.0; // 90.0
+const double ServerParam::SIDE_DASH_RATE = 0.25;
+const double ServerParam::BACK_DASH_RATE = 0.5;
+const double ServerParam::MAX_DASH_POWER = 100.0;
+const double ServerParam::MIN_DASH_POWER = -100.0;
 
 /*
   It is necessary to check out whether server is NEW_QSTEP mode
@@ -593,10 +605,23 @@ ServerParam::setDefaultParam()
     M_max_back_tackle_power = DEFAULT_MAX_BACK_TACKLE_POWER;
     M_player_speed_max_min = DEFAULT_PLAYER_SPEED_MAX_MIN;
     M_extra_stamina = DEFAULT_EXTRA_STAMINA;
-    M_synch_see_offset = 30;
+    M_synch_see_offset = DEFAULT_SYNCH_SEE_OFFSET;
 
     M_max_monitors = -1;
 
+    // 12.1.3
+    M_extra_half_time = EXTRA_HALF_TIME;
+
+    // 13.0.0
+    M_stamina_capacity = STAMINA_CAPACITY;
+    M_max_dash_angle = MAX_DASH_ANGLE;
+    M_min_dash_angle = MIN_DASH_ANGLE;
+    M_dash_angle_step = DASH_ANGLE_STEP;
+    M_side_dash_rate = SIDE_DASH_RATE;
+    M_back_dash_rate = BACK_DASH_RATE;
+    M_max_dash_power = MAX_DASH_POWER;
+    M_min_dash_power = MIN_DASH_POWER;
+
     setAdditionalParam();
 }
 
@@ -608,12 +633,12 @@ void
 ServerParam::setAdditionalParam()
 {
     M_kickable_area = M_kickable_margin + M_ball_size + M_player_size;
-    M_catchable_area = std::sqrt( std::pow( catchAreaWidth() * 0.5, 2.0 )
-                                  + std::pow( catchAreaLength(), 2.0 ) );
+    M_catchable_area = std::sqrt( std::pow( catchAreaWidth() * 0.5, 2 )
+                                  + std::pow( catchAreaLength(), 2 ) );
     M_control_radius_width = M_control_radius - M_player_size;
 
     // real speed max
-    double accel_max = maxPower() * defaultDashPowerRate() * defaultEffortMax();
+    double accel_max = maxDashPower() * defaultDashPowerRate() * defaultEffortMax();
     M_real_speed_max = accel_max / ( 1.0 - defaultPlayerDecay() );  // sum inf geom series
     if ( M_real_speed_max > defaultPlayerSpeedMax() )
     {
@@ -866,12 +891,23 @@ ServerParam::createMap()
 
         ( "ball_stuck_area", "", &M_ball_stuck_area )
         ( "coach_msg_file", "", &M_coach_msg_file )
-
+        // 12
         ( "max_tackle_power", "", &M_max_tackle_power )
         ( "max_back_tackle_power", "", &M_max_back_tackle_power )
         ( "player_speed_max_min", "", &M_player_speed_max_min )
         ( "extra_stamina", "", &M_extra_stamina )
         ( "synch_see_offset", "", &M_synch_see_offset )
+        // 12.1.3
+        ( "extra_half_time", "",  &M_extra_half_time )
+        // 13.0.0
+        ( "stamina_capacity", "", &M_stamina_capacity )
+        ( "max_dash_angle", "", &M_max_dash_angle )
+        ( "min_dash_angle", "", &M_min_dash_angle )
+        ( "dash_angle_step", "", &M_dash_angle_step )
+        ( "side_dash_rate", "",  &M_side_dash_rate )
+        ( "back_dash_rate", "", &M_back_dash_rate )
+        ( "max_dash_power", "", &M_max_dash_power )
+        ( "min_dash_power", "", &M_min_dash_power )
 
         ( "max_monitors", "", &M_max_monitors )
         ( "min_catch_probability", "", &M_min_catch_probability )
@@ -1247,16 +1283,16 @@ ServerParam::convertFrom( const rcg::server_params_t & from )
 
     // 12.0.0
     tmp = rcg::nltohd( from.max_tackle_power );
-    if ( 0.0 < tmp && std::fabs( tmp ) < 1000.0 ) M_max_tackle_power = tmp;
+    if ( 0.0 < tmp && std::fabs( tmp ) < 200.0 ) M_max_tackle_power = tmp;
     tmp = rcg::nltohd( from.max_back_tackle_power );
-    if ( 0.0 < tmp && std::fabs( tmp ) < 1000.0 ) M_max_back_tackle_power = tmp;
+    if ( 0.0 < tmp && std::fabs( tmp ) < 200.0 ) M_max_back_tackle_power = tmp;
 
     tmp = rcg::nltohd( from.tackle_dist );
-    if ( 0.0 <= tmp && std::fabs( tmp ) < 1000.0 ) M_tackle_dist = tmp;
+    if ( 0.0 <= tmp && std::fabs( tmp ) < 3.0 ) M_tackle_dist = tmp;
     tmp = rcg::nltohd( from.tackle_back_dist );
-    if ( 0.0 <= tmp && std::fabs( tmp ) < 1000.0 ) M_tackle_back_dist = tmp;
+    if ( 0.0 <= tmp && std::fabs( tmp ) < 1.0 ) M_tackle_back_dist = tmp;
     tmp = rcg::nltohd( from.tackle_width );
-    if ( 0.0 < tmp && std::fabs( tmp ) < 1000.0 ) M_tackle_width = tmp;
+    if ( 0.0 < tmp && std::fabs( tmp ) < 2.0 ) M_tackle_width = tmp;
 
     //
     M_start_goal_l = rcg::nstohi( from.start_goal_l );
@@ -1458,16 +1494,7 @@ ServerParam::toStr() const
           ++it )
     {
         os << '(' << it->second->longName() << ' ';
-        if ( it->second->isString() )
-        {
-            os << '"';
-            it->second->printValue( os );
-            os << '"';
-        }
-        else
-        {
-            it->second->printValue( os );
-        }
+        it->second->printValue( os );
         os << ')';
     }
 
@@ -1476,4 +1503,22 @@ ServerParam::toStr() const
     return os.str();
 }
 
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+double
+ServerParam::dashDirRate( const double & dir ) const
+{
+    double d = discretizeDashAngle( normalizeDashAngle( dir ) );
+    double r = std::fabs( d ) > 90.0
+        ? backDashRate() - ( ( backDashRate() - sideDashRate() )
+                             * ( 1.0 - ( std::fabs( d ) - 90.0 ) / 90.0 )
+                             )
+        : sideDashRate() + ( ( 1.0 - sideDashRate() )
+                             * ( 1.0 - std::fabs( d ) / 90.0 ) )
+        ;
+    return std::min( std::max( 1.0e-5, r ), 1.0 );
+}
+
 }
