@@ -2,7 +2,7 @@
 
 /*!
 	\file formation.h
-	\brief formation data classes Header File.
+	\brief abstract formation classes Header File.
 */
 
 /*
@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,7 +32,9 @@
 #ifndef RCSC_FORMATION_FORMATION_H
 #define RCSC_FORMATION_FORMATION_H
 
+#include <rcsc/formation/sample_data.h>
 #include <rcsc/geom/vector_2d.h>
+#include <rcsc/factory.h>
 
 #include <boost/shared_ptr.hpp>
 
@@ -50,52 +52,77 @@ namespace rcsc {
 class Formation {
 public:
 
+    typedef boost::shared_ptr< Formation > Ptr; //<! pointer type
+    typedef boost::shared_ptr< const Formation > ConstPtr; //<! const pointer type
+    typedef Ptr (*Creator)(); //!< creator function
+    typedef rcss::Factory< Creator, std::string > Creators; //!< creator function holder
+
+
+    /*!
+      \enum SideType
+      \brief enumeration for the side type in formation
+     */
     enum SideType {
         SIDE = -1, //!< original type, consider all region
-        MIRROR = 1, //!< copy type, refere SIDE
+        SYMMETRY = 1, //!< symmetry type, this type refers SIDE
         CENTER = 0, //!< original type, consider half region
     };
 
     /*!
-      \brief this struct is used to record the field status
-      as a training data
-    */
-    struct Snapshot {
-        Vector2D ball_; //!< ball position
-        std::vector< Vector2D > players_; //!< players' position
+      \brief creator holder singleton
+      \return reference to the creator holder instance
+     */
+    static
+    Creators & creators();
 
-        Snapshot()
-          {
-              players_.reserve( 11 );
-          }
 
-        Snapshot( const Vector2D & ball,
-                  const std::vector< Vector2D > & players )
-            : ball_( ball )
-            , players_( players )
-          { }
-    };
+    /*!
+      \brief create a formation instance based on the input name.
+      \param name formation type name
+      \return smart pointer to the formation instance
+     */
+    static
+    Ptr create( const std::string & name );
+
+    /*!
+      \brief create a formation instance based on the input stream.
+      \param is reference to the input stream (usually fstream).
+      \return smart pointer to the formation instance
+     */
+    static
+    Ptr create( std::istream & is );
 
 protected:
 
     /*!
-      mirror number.holder
-      <0 negative number means this player is original SIDE type.
-      >0 positive numver means that this player is MIRROR type and
-      referes other player.
-      ==0 Zero means that this player is CENTER type.
-    */
-    int M_mirror_number[11];
+      \brief data format version.
+     */
+    int M_version;
 
+    /*!
+      \brief symmetry number holder.
+      negative number means this player is original SIDE type.
+      positive numver means that this player is SYMMETRY type and
+      referes other player.
+      Zero means that this player is CENTER type.
+    */
+    int M_symmetry_number[11];
+
+    /*!
+      \brief training data holder.
+     */
+    formation::SampleDataSet::Ptr M_samples;
 
 public:
 
     /*!
-      \brief initialize mirror number matrix by -1
+      \brief initialize symmetry number matrix by -1
     */
     Formation();
 
-
+    /*!
+      \brief destructor.
+     */
     virtual
     ~Formation()
       { }
@@ -105,15 +132,51 @@ public:
       \return snapshot variable for the initial state(ball pos=(0,0)).
     */
     virtual
-    Snapshot createDefaultParam() = 0;
+    void createDefaultData() = 0;
 
     /*!
-      \brief get the name of this formation
+      \brief get data format version.
+      \return data format version.
+     */
+    int version() const
+      {
+          return M_version;
+      }
+
+    /*!
+      \brief get the formation method type name.
       \return name string
     */
     virtual
     std::string methodName() const = 0;
 
+    /*!
+      \brief get sample point set.
+      \return shared pointer to the training data object.
+     */
+    formation::SampleDataSet::Ptr samples()
+      {
+          return M_samples;
+      }
+
+    /*!
+      \brief get sample point set.
+      \return shared pointer to the training data object.
+     */
+    formation::SampleDataSet::ConstPtr samples() const
+      {
+          return M_samples;
+      }
+
+    /*!
+      \brief set new sample point set.
+      \param data pointer to the new data instance.
+     */
+    void setSamples( formation::SampleDataSet::Ptr samples )
+      {
+          M_samples = samples;
+      }
+
     /*!
       \brief check if player is SIDE type or not
       \param unum player's number
@@ -122,7 +185,7 @@ public:
     bool isSideType( const int unum ) const
       {
           if ( unum < 1 || 11 < unum ) return false;
-          return ( M_mirror_number[unum - 1] < 0 );
+          return ( M_symmetry_number[unum - 1] < 0 );
       }
 
     /*!
@@ -133,7 +196,7 @@ public:
     bool isCenterType( const int unum ) const
       {
           if ( unum < 1 || 11 < unum ) return false;
-          return ( M_mirror_number[unum - 1] == 0 );
+          return ( M_symmetry_number[unum - 1] == 0 );
       }
 
     /*!
@@ -141,22 +204,22 @@ public:
       \param unum player's number
       \return true or false
     */
-    bool isMirrorType( const int unum ) const
+    bool isSymmetryType( const int unum ) const
       {
           if ( unum < 1 || 11 < unum ) return false;
-          return ( M_mirror_number[unum - 1] > 0 );
+          return ( M_symmetry_number[unum - 1] > 0 );
       }
 
     /*!
       \brief get symmetry reference number of the specified player.
       \param unum target player's number
-      \retrun number that player refers, if player is MIRROR type.
+      \return number that player refers, if player is SYMMETRY type.
       otherwise 0 or -1.
     */
-    int getMirrorNumber( const int unum ) const
+    int getSymmetryNumber( const int unum ) const
       {
           if ( unum < 1 || 11 < unum ) return 0;
-          return M_mirror_number[unum - 1];
+          return M_symmetry_number[unum - 1];
       }
 
     //--------------------------------------------------------------
@@ -164,13 +227,13 @@ public:
     /*!
       \brief set player's role data. if necessary, new parameter is created.
       \param unum status changed player's uniform number
-      \param mirror_unum 0 means type is CENTER, <0 means type is SIDE,
-      >0 means type is MIRROR
+      \param symmetry_unum 0 means type is CENTER, <0 means type is SIDE,
+      >0 means type is SYMMETRY
       \param role_name new role name string
       \return result of the registration
     */
     bool updateRole( const int unum,
-                     const int mirror_unum,
+                     const int symmetry_unum,
                      const std::string & role_name );
 
 protected:
@@ -211,10 +274,11 @@ protected:
     /*!
       \brief set symmetry player info
       \param unum changed player's number
-      \param mirror_unum refered player number
+      \param symmetry_unum refered player number
     */
-    bool setMirrorType( const int unum,
-                        const int mirror_unum );
+    bool setSymmetryType( const int unum,
+                          const int symmetry_unum,
+                          const std::string & role_name );
 
 public:
 
@@ -235,7 +299,8 @@ public:
     virtual
     Vector2D getPosition( const int unum,
                           const Vector2D & focus_point ) const = 0;
-    /*
+
+    /*!
       \brief get all positions for the current focus point
       \param focus_point current focus point, usually ball position
       \param positions contaner to store the result
@@ -246,52 +311,86 @@ public:
 
     /*!
       \brief update formation paramter using training data set
-      \param train_data training data container
     */
     virtual
-    void train( const std::list< Snapshot > & train_data ) = 0;
+    void train() = 0;
 
     /*!
-      \brief restore data from the input stream.
+      \brief read all data from the input stream.
       \param is reference to the input stream.
-      \return parsing result
+      \return result status.
     */
-    virtual
-    bool read( std::istream & is ) = 0;
+    bool read( std::istream & is );
 
     /*!
-      \brief put data to the output stream.
+      \brief put all data to the output stream.
       \param os reference to the output stream
       \return reference to the output stream
     */
-    virtual
-    std::ostream & print( std::ostream & os ) const = 0;
+    std::ostream & print( std::ostream & os ) const;
 
 
 protected:
 
-    /*!
-      \brief check formation type name from the current stream pos.
-      \param is reference to the input stream
-      \return if success, the number of read lines will be returned.
-      if failed, returned -1.
-    */
-    int readName( std::istream & is );
+    //
+    // read
+    //
+
 
     /*!
-      \brief put formation type name to the output stream
+      \brief read header information (formation type name, format version...)
+      \param is reference to the input stream
+      \return result status.
+    */
+    virtual
+    bool readHeader( std::istream & is );
+
+    /*!
+      \brief read conf data from the input stream.
+      \param is reference to the input stream.
+      \return result status.
+    */
+    virtual
+    bool readConf( std::istream & is ) = 0;
+
+    /*!
+      \brief read sample point data from the input stream.
+      \param is reference to the input stream.
+      \return result status.
+    */
+    virtual
+    bool readSamples( std::istream & is );
+
+
+    //
+    // print
+    //
+
+    /*!
+      \brief put header information to the output stream
       \param os reference to the output stream
       \return reference to the output stream
     */
-    std::ostream & printName( std::ostream & os ) const;
+    virtual
+    std::ostream & printHeader( std::ostream & os ) const;
 
+    /*!
+      \brief put conf data to the output stream.
+      \param os reference to the output stream
+      \return reference to the output stream
+    */
+    virtual
+    std::ostream & printConf( std::ostream & os ) const = 0;
+
+    /*!
+      \brief put sample point data to the output stream.
+      \param os reference to the output stream
+      \return reference to the output stream
+    */
+    virtual
+    std::ostream & printSamples( std::ostream & os ) const;
 };
 
-/*!
-  \brief type of the formation pointer
- */
-typedef boost::shared_ptr< Formation > FormationPtr;
-
 }
 
 #endif
