@@ -1,8 +1,8 @@
 // -*-c++-*-
 
 /*!
-	\file formation_dt.h
-	\brief formation data classes using Delaunay Triangulation Header File.
+  \file formation_dt.h
+  \brief formation data classes using Delaunay Triangulation Header File.
 */
 
 /*
@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -46,60 +46,7 @@ class FormationDT
     : public Formation {
 public:
 
-    /*!
-      \struct Param
-      \brief training data object
-     */
-    struct Param {
-        Vector2D ball_; //!< ball position = focus point
-        std::vector< Vector2D > players_; //!< all players' desired position
-
-        /*!
-          \brief just allocate memory
-        */
-        Param()
-            : ball_( 0.0, 0.0 )
-            , players_( 11 )
-          { }
-
-        /*!
-          \brief initialize with training data snapshot
-        */
-        Param( const Snapshot & data );
-
-        /*!
-          \brief initialize with specified data
-          \param ball ball position
-          \param players players' position container
-        */
-        Param( const Vector2D & ball,
-               const std::vector< Vector2D > & players )
-            : ball_( ball )
-            , players_( players )
-          { }
-
-        /*!
-          \brief initialize with specified data
-          \param ball ball position
-          \param players players' position container
-          \return const reference to itself
-        */
-        const
-        Param & assign( const Vector2D & ball,
-                        const std::vector< Vector2D > & players )
-          {
-              ball_ = ball;
-              players_ = players;
-              return *this;
-          }
-
-        /*!
-          \brief get the specified player's position
-          \param unum player number
-          \return position value
-        */
-        Vector2D getPosition( const int unum ) const;
-    };
+    static const std::string NAME; //!< type name
 
 private:
 
@@ -107,7 +54,7 @@ private:
     std::string M_role_name[11];
 
     //! set of desired positins used by delaunay triangulation & linear interpolation
-    std::vector< Param > M_param;
+    std::vector< formation::SampleData > M_sample_vector;
 
     //! delaunay triangulation
     DelaunayTriangulation M_triangulation;
@@ -119,15 +66,6 @@ public:
     */
     FormationDT();
 
-    /*!
-      \brief simple accessor.
-      \return const reference to the param instance
-     */
-    const
-    std::vector< Param > & params() const
-      {
-          return M_param;
-      }
 
     /*!
       \brief static method. get formation method name
@@ -136,7 +74,8 @@ public:
     static
     std::string name()
       {
-          return std::string( "DelaunayTriangulation" );
+          //return std::string( "DelaunayTriangulation" );
+          return NAME;
       }
 
     /*!
@@ -144,9 +83,9 @@ public:
       \return new object
     */
     static
-    Formation * create()
+    Formation::Ptr create()
       {
-          return ( new FormationDT );
+          return Formation::Ptr( new FormationDT() );
       }
 
     /*!
@@ -161,6 +100,13 @@ public:
 
     //--------------------------------------------------------------
 
+    /*!
+      \brief create default formation. assign role and initial positions.
+      \return snapshow variable for the initial state(ball pos=(0,0)).
+    */
+    virtual
+    void createDefaultData();
+
     /*!
       \brief get the name of this formation
       \return name string
@@ -171,13 +117,6 @@ public:
           return FormationDT::name();
       }
 
-    /*!
-      \brief create default formation. assign role and initial positions.
-      \return snapshow variable for the initial state(ball pos=(0,0)).
-    */
-    virtual
-    Snapshot createDefaultParam();
-
 protected:
     /*!
       \brief create new role parameter.
@@ -212,13 +151,13 @@ public:
     /*!
       \brief get position for the current focus point
       \param unum player number
-      \param forcus_point current focus point, usually ball position.
+      \param focus_point current focus point, usually ball position.
     */
     virtual
     Vector2D getPosition( const int unum,
                           const Vector2D & focus_point ) const;
 
-    /*
+    /*!
       \brief get all positions for the current focus point
       \param focus_point current focus point, usually ball position
       \param positions contaner to store the result
@@ -229,26 +168,9 @@ public:
 
     /*!
       \brief update formation paramter using training data set
-      \param train_data training data container
     */
     virtual
-    void train( const std::list< Snapshot > & train_data );
-
-    /*!
-      \brief restore data from the input stream.
-      \param is reference to the input stream.
-      \return parsing result
-    */
-    virtual
-    bool read( std::istream & is );
-
-    /*!
-      \brief put data to the output stream.
-      \param os reference to the output stream
-      \return reference to the output stream
-    */
-    virtual
-    std::ostream & print( std::ostream & os ) const;
+    void train();
 
 private:
 
@@ -256,20 +178,122 @@ private:
                           const Vector2D & focus_point,
                           const DelaunayTriangulation::Triangle * tri ) const;
 
+
+protected:
+
     /*!
-      \brief restore role assignment from the input stream
+      \brief restore conf data from the input stream.
+      \param is reference to the input stream.
+      \return parsing result
+    */
+    virtual
+    bool readConf( std::istream & is );
+
+    /*!
+      \brief read sample point data from the input stream.
+      \param is reference to the input stream.
+      \return result status.
+    */
+    virtual
+    bool readSamples( std::istream & is );
+
+    /*!
+      \brief put data to the output stream.
+      \param os reference to the output stream
+      \return reference to the output stream
+    */
+    virtual
+    std::ostream & printConf( std::ostream & os ) const;
+
+    /*!
+      \brief put sample point data to the output stream.
+      \param os reference to the output stream
+      \return reference to the output stream
+    */
+    virtual
+    std::ostream & printSamples( std::ostream & os ) const;
+
+private:
+
+    /*!
+      \brief read v1 format data.
+      \param is reference to the input stream
+      \return parsing result.
+    */
+    bool readV1( std::istream & is );
+
+    /*!
+      \brief (v1 format) restore role assignment from the input stream
       \param is reference to the input stream
       \return parsing result
     */
     bool readRoles( std::istream & is );
 
     /*!
-      \brief restore kernel point and sample data from the input stream
+      \brief (v1 format) restore kernel point and sample data from the input stream
       \param is reference to the input stream
       \return parsing result
     */
-    bool readSamples( std::istream & is );
+    bool readVertices( std::istream & is );
 
+
+    /*!
+      \brief read v1 format data.
+      \param is reference to the input stream
+      \return parsing result.
+    */
+    bool readV2( std::istream & is );
+
+    /*!
+      \brief (v2 format) restore role assignment from the input stream
+      \param is reference to the input stream
+      \return parsing result
+    */
+    bool readRolesV2( std::istream & is );
+
+    /*!
+      \brief (v2 format) restore vertex data set from the input stream
+      \param is reference to the input stream
+      \return parsing result
+    */
+    bool readVerticesV2( std::istream & is );
+
+    /*!
+      \brief (v2 format) restore one vertex data from the input stream.
+      \param is reference to the input stream.
+      \param data_index index number of this data.
+      \return parsing result
+    */
+    bool readOneDataV2( std::istream & is,
+                        const int data_index );
+
+    /*!
+      \brief (v1 format) print all data to the output stream
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printV1( std::ostream & os ) const;
+
+    /*!
+      \brief (v2 format) print all data to the output stream
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printV2( std::ostream & os ) const;
+
+    /*!
+      \brief (v2 format) print role data to the output stream
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printRolesV2( std::ostream & os ) const;
+
+    /*!
+      \brief (v2 format) print positional data to the output stream
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & printDataV2( std::ostream & os ) const;
 };
 
 }
