@@ -1,8 +1,8 @@
 // -*-c++-*-
 
 /*!
-	\file formation_uva.h
-	\brief UvA Trilearn type formation method classes Header File.
+  \file formation_uva.h
+  \brief UvA Trilearn type formation method classes Header File.
 */
 
 /*
@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -48,6 +48,8 @@ class FormationUvA
     : public Formation {
 public:
 
+    static const std::string NAME; //!< type name
+
     /*!
       \class RoleParam
       \brief This class contains information for one individual role.
@@ -172,9 +174,8 @@ public:
 private:
     std::string M_role_names[11]; //!< role names
     Vector2D M_home_pos[11]; //!< home position for roles
-    //! key: role name, value role parameter
-    std::map< std::string, RoleParam > M_role_params;
-    double M_max_y_percentage;
+    std::map< std::string, RoleParam > M_role_params; //! key: role name, value role parameter
+    double M_max_y_percentage; //!< the rate of maximum y coordinate in the field coordinate system
 public:
     /*!
       \brief just call the base class constructor
@@ -188,7 +189,8 @@ public:
     static
     std::string name()
       {
-          return std::string( "UvA" );
+          //return std::string( "UvA" );
+          return NAME;
       }
 
     /*!
@@ -196,13 +198,20 @@ public:
       \return pointer to the new instance
      */
     static
-    Formation * create()
+    Formation::Ptr create()
       {
-          return ( new FormationUvA );
+          return Formation::Ptr( new FormationUvA() );
       }
 
     //--------------------------------------------------------------
 
+    /*!
+      \brief create default formation. assign role and initial positions.
+      \return snapshow variable for the initial stat(ball pos=(0,0)).
+     */
+    virtual
+    void createDefaultData();
+
     /*!
       \brief get the name of this formation
       \return name string
@@ -213,13 +222,6 @@ public:
           return FormationUvA::name();
       }
 
-    /*!
-      \brief create default formation. assign role and initial positions.
-      \return snapshow variable for the initial stat(ball pos=(0,0)).
-     */
-    virtual
-    Snapshot createDefaultParam();
-
     /*!
       \brief get the maximal y percentage of players' position
       \return y percentage value [0,1]
@@ -298,15 +300,17 @@ public:
       \param train_data training data container
      */
     virtual
-    void train( const std::list< Snapshot > & train_data );
+    void train();
+
+protected:
 
     /*!
-      \brief restore data from the input stream.
+      \brief restore conf data from the input stream.
       \param is reference to the input stream.
-      \return parsing result
-     */
+      \return pasing result.
+    */
     virtual
-    bool read( std::istream & is );
+    bool readConf( std::istream & is );
 
     /*!
       \brief put all variables to output stream
@@ -314,7 +318,7 @@ public:
       \return  reference to the output stream
      */
     virtual
-    std::ostream & print( std::ostream & os ) const;
+    std::ostream & printConf( std::ostream & os ) const;
 
 private:
 
