@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -170,7 +170,7 @@ public:
     "penalty_draw"
     */
 
-    typedef std::pair< Type, rcsc::SideID > Pair;
+    typedef std::pair< Type, rcsc::SideID > Pair; //!< alias of the pair of playmode type and side type
 
 private:
 
