@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -36,11 +36,12 @@
 #include "line_2d.h"
 
 #include <iostream>
+#include <limits>
 
 namespace rcsc {
 
 const double Line2D::EPSILON = 1.0e-5;
-const double Line2D::ERROR_VAL = HUGE_VAL;
+const double Line2D::ERROR_VALUE = std::numeric_limits< double >::max();
 
 /*-------------------------------------------------------------------*/
 /*!
