@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -47,25 +47,27 @@ namespace rcsc {
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 Polygon2D::Polygon2D()
     : M_vertex()
 {
+
 }
 
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 Polygon2D::Polygon2D( const std::vector< Vector2D > & v )
     : M_vertex( v )
 {
+
 }
 
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 const Polygon2D &
 Polygon2D::assign()
 {
@@ -76,7 +78,7 @@ Polygon2D::assign()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 const Polygon2D &
 Polygon2D::assign( const std::vector< Vector2D > & v )
 {
@@ -87,7 +89,7 @@ Polygon2D::assign( const std::vector< Vector2D > & v )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 Polygon2D::addVertex( const Vector2D & p )
 {
@@ -97,7 +99,7 @@ Polygon2D::addVertex( const Vector2D & p )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 const std::vector< Vector2D > &
 Polygon2D::vertex() const
 {
@@ -107,7 +109,7 @@ Polygon2D::vertex() const
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 std::vector< Vector2D > &
 Polygon2D::vertex()
 {
@@ -117,7 +119,7 @@ Polygon2D::vertex()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 Rect2D
 Polygon2D::getBoundingBox() const
 {
@@ -163,7 +165,7 @@ Polygon2D::getBoundingBox() const
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 Vector2D
 Polygon2D::xyCenter() const
 {
@@ -173,7 +175,7 @@ Polygon2D::xyCenter() const
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 Polygon2D::contains( const Vector2D & p,
                      const bool allow_on_segment ) const
@@ -184,7 +186,10 @@ Polygon2D::contains( const Vector2D & p,
     }
     else if ( M_vertex.size() == 1 )
     {
-        return allow_on_segment && (M_vertex[0] == p);
+        return allow_on_segment
+            //&& ( M_vertex[0].equals( p ) );
+            && ( M_vertex[0] == p );
+
     }
 
 
@@ -230,7 +235,10 @@ Polygon2D::contains( const Vector2D & p,
             }
         }
 
-        if ( allow_on_segment && p == p0 )
+        if ( allow_on_segment
+            //&& p.equalsStrictly( p0 ) )
+            && p == p0 )
+
         {
             return true;
         }
@@ -238,7 +246,7 @@ Polygon2D::contains( const Vector2D & p,
         if ( line.existIntersection( Segment2D( p0, p1 ) ) )
         {
             if ( p0.y == p.y
-              || p1.y == p.y )
+                 || p1.y == p.y )
             {
                 if ( p0.y == p.y )
                 {
@@ -262,7 +270,7 @@ Polygon2D::contains( const Vector2D & p,
                     continue;
                 }
                 else if ( p0.y < p.y
-                       || p1.y < p.y )
+                          || p1.y < p.y )
                 {
                     continue;
                 }
@@ -283,7 +291,7 @@ Polygon2D::contains( const Vector2D & p,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 double
 Polygon2D::dist( const Vector2D & p,
                  bool check_as_plane ) const
@@ -334,7 +342,7 @@ Polygon2D::dist( const Vector2D & p,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 double
 Polygon2D::area() const
 {
@@ -344,7 +352,7 @@ Polygon2D::area() const
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 double
 Polygon2D::signedArea2() const
 {
@@ -371,7 +379,7 @@ Polygon2D::signedArea2() const
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 Polygon2D::isCounterclockwise() const
 {
@@ -381,7 +389,7 @@ Polygon2D::isCounterclockwise() const
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 Polygon2D::isClockwise() const
 {
@@ -542,7 +550,7 @@ public:
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 Polygon2D
 Polygon2D::getScissoredConnectedPolygon( const Rect2D & r ) const
 {
