@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -308,7 +308,7 @@ gzfilebuf::makeModeString( std::ios_base::openmode mode,
               && level <= BEST_COMPRESSION )
     {
         char lvstr[4];
-        std::snprintf( lvstr, 4, "%d", level );
+        snprintf( lvstr, 4, "%d", level );
         mode_str += lvstr;
     }
     else
