@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -96,6 +96,21 @@ sign( const double & x )
     return x > 0.0 ? 1.0 : -1.0;
 }
 
+/*-------------------------------------------------------------------*/
+/*!
+  \brief round a floating point number with specified precision
+  \param value value to be rounded
+  \param prec round precision
+  \return rounded value
+*/
+inline
+double
+round( const double & value,
+       const double & prec )
+{
+    return rint( value / prec ) * prec;
+}
+
 /*-------------------------------------------------------------------*/
 /*!
   \brief Quadratic Formulation (ax^2 + bx + c = 0)
