@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -77,6 +77,8 @@ protected:
     int M_face_count; //!< face angle accuracy
 
     double M_dist_from_ball; //!< distance from ball
+    AngleDeg M_angle_from_ball; //!< angle from ball
+    double M_dist_from_self; //!< distance from self
 
 public:
 
@@ -132,6 +134,20 @@ public:
           return 0;
       }
 
+    /*!
+      \brief check if player is tackling or not
+      \return checked result
+     */
+    virtual
+    bool isTackling() const = 0;
+
+    /*!
+      \brief update player type id
+      \param type new player type id
+     */
+    virtual
+    void setPlayerType( const int type );
+
     // ------------------------------------------
 
     /*!
@@ -189,12 +205,6 @@ public:
           return M_player_type;
       }
 
-    /*!
-      \brief update player type id
-      \param type new player type id
-     */
-    void setPlayerType( const int type );
-
     /*!
       \brief get global position
       \return const reference to the point object
@@ -338,6 +348,34 @@ public:
           return M_dist_from_ball;
       }
 
+    /*!
+      \brief get angle from ball
+      \return absolute angle value from ball
+    */
+    const
+    AngleDeg &
+    angleFromBall() const
+      {
+          return M_angle_from_ball;
+      }
+
+    /*!
+      \brief get distance from self
+      \return distance value from self
+    */
+    const
+    double & distFromSelf() const
+      {
+          return M_dist_from_self;
+      }
+
+    /*!
+      \brief get current estimated kick power rate
+      \return calculated kick rate value
+    */
+    virtual
+    double kickRate() const;
+
     /*!
       \brief estimate reach point
       \param n_step this method estimates ball point after this steps
@@ -376,7 +414,7 @@ public:
 };
 
 //! typedef of the AbstractPlaeyrObject container
-typedef std::vector< AbstractPlayerObject * > AbstractPlayerCont;
+typedef std::vector< const AbstractPlayerObject * > AbstractPlayerCont;
 
 }
 
