@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,7 +32,7 @@
 #ifndef RCSC_PLAYER_AUDIO_SENSOR_H
 #define RCSC_PLAYER_AUDIO_SENSOR_H
 
-#include <rcsc/player/audio_message.h>
+#include <rcsc/common/audio_message.h>
 
 #include <rcsc/game_time.h>
 #include <rcsc/types.h>
@@ -43,6 +43,7 @@
 #include <string>
 #include <map>
 #include <list>
+#include <vector>
 
 namespace rcsc {
 
@@ -57,8 +58,11 @@ class FreeformParser;
 class AudioSensor {
 private:
 
+    //! typedef of the say message parser container
+    typedef std::map< char, boost::shared_ptr< SayMessageParser > > ParserMap;
+
     //! player message parsers
-    std::map< char, boost::shared_ptr< SayMessageParser > > M_say_message_parsers;
+    ParserMap M_say_message_parsers;
 
     //! freeform message parsers
     boost::shared_ptr< FreeformParser > M_freeform_parser;
@@ -103,13 +107,13 @@ public:
       \brief remove registered parser object
       \param header say message header character
      */
-    void removeSayMessageParser( const char header );
+    void removeParser( const char header );
 
     /*!
-      \brief add new freeform message parer.
+      \brief set new freeform message parer.
       \param parser shared_ptr of player message parser instance
      */
-    void addParser( boost::shared_ptr< FreeformParser > parser );
+    void setFreeformParser( boost::shared_ptr< FreeformParser > parser );
 
     /*!
       \brief analyze other player's audio message
