@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -53,6 +53,7 @@ public:
     ViewWidth M_view_width; //!< sensed view width
     double M_stamina; //!< sensed stamina value
     double M_effort; //!< sensed effort value
+    double M_stamina_capacity; //!< sensed stamina capacity
     double M_speed_mag; //!< sensed speed magnitude. this is quantized by 0.01.
     double M_speed_dir_relative; //!< speed dir. this is relative to face angle.
     double M_neck_relative; //!< neck angle. this is relative to body angle
@@ -205,6 +206,16 @@ public:
           return M_effort;
       }
 
+    /*!
+      \brief get analized stamina capacity value
+      \return stamina capacity value
+     */
+    const
+    double & staminaCapacity() const
+      {
+          return M_stamina_capacity;
+      }
+
     /*!
       \brief get analyzed speed value
       \return scalar value of velocity
@@ -245,7 +256,7 @@ public:
       }
 
     /*!
-      \brief get analyzed kick count
+      \brief get analyzed dash count
       \return count of performed kick command
     */
     int dashCount() const
@@ -400,7 +411,7 @@ public:
       }
 
     /*!
-      \brier get the information wheter the agent receive does not collide.
+      \brief get the information wheter the agent receive does not collide.
       \return true if this body sensor has 'none' collision information.
      */
     bool noneCollided() const
