@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -34,10 +34,13 @@
 #endif
 
 #include "debug_client.h"
+
+#include "action_effector.h"
 #include "world_model.h"
 #include "player_object.h"
+#include "say_message_builder.h"
 
-#include <rcsc/common/logger.h>
+#include <rcsc/common/audio_memory.h>
 #include <rcsc/net/udp_socket.h>
 
 #include <algorithm>
@@ -108,9 +111,19 @@ public:
                    << ')';
           }
 
-          M_os << " (c \""
-               << ( p.goalie() ? 'G' : M_tag )
-               << p.posCount()
+          M_os << " (c \"";
+
+          if  ( p.goalie() )
+          {
+              M_os << "G:";
+          }
+
+          if ( p.unum() != Unum_Unknown )
+          {
+              M_os << 'u' << p.unumCount();
+          }
+
+          M_os << 'p' << p.posCount()
                << 'v' << p.velCount();
 
           if ( p.velCount() <= 100 )
@@ -119,7 +132,18 @@ public:
                    << ' ' << ROUND( p.vel().y, 0.1 )
                    << ')';
           }
-          M_os << 'f' << p.faceCount() << "\"))";
+          M_os << 'f' << p.faceCount();
+
+          if ( p.isTackling() )
+          {
+              M_os << "(t)";
+          }
+          else if ( p.kicked() )
+          {
+              M_os << "(k)";
+          }
+
+          M_os << "\"))";
       }
 };
 
@@ -132,13 +156,13 @@ public:
     LinePrinter( std::ostream & os )
         : M_os( os )
       { }
-    void operator()( const std::pair< Vector2D, Vector2D > & line )
+    void operator()( const Segment2D & line )
       {
           M_os << " (line "
-               << ROUND( line.first.x, 0.001 ) << ' '
-               << ROUND( line.first.y, 0.001 ) << ' '
-               << ROUND( line.second.x, 0.001 ) << ' '
-               << ROUND( line.second.y, 0.001 ) << ')';
+               << ROUND( line.origin().x, 0.001 ) << ' '
+               << ROUND( line.origin().y, 0.001 ) << ' '
+               << ROUND( line.terminal().x, 0.001 ) << ' '
+               << ROUND( line.terminal().y, 0.001 ) << ')';
       }
 };
 
@@ -209,13 +233,17 @@ DebugClient::DebugClient()
     , M_connected( false )
     , M_write_mode( false )
     , M_main_buffer( "" )
-    , M_target_unum( 0 )
+    , M_target_unum( Unum_Unknown )
     , M_target_point( Vector2D::INVALIDATED )
     , M_message( "" )
 {
     M_main_buffer.reserve( 8192 );
     M_message.reserve( 8192 );
-    M_lines.reserve( 100 );
+
+    M_lines.reserve( MAX_LINE );
+    M_triangles.reserve( MAX_TRIANGLE );
+    M_rectangles.reserve( MAX_RECT );
+    M_circles.reserve( MAX_CIRCLE );
 }
 
 /*-------------------------------------------------------------------*/
@@ -235,14 +263,12 @@ void
 DebugClient::connect( const std::string & hostname,
                       const int port )
 {
-    M_socket
-        = boost::shared_ptr< UDPSocket >
-        ( new UDPSocket( hostname.c_str(), port ) );
+    M_socket = boost::shared_ptr< UDPSocket >( new UDPSocket( hostname.c_str(), port ) );
 
     if ( ! M_socket
          || M_socket->fd() == -1 )
     {
-        std::cout << "cannot connect to soccer viewer host=["
+        std::cout << "cannot connect to the debug server host=["
                   << hostname << "] port=[" << port << "]"
                   << std::endl;
         if ( M_socket ) M_socket.reset();
@@ -257,14 +283,37 @@ DebugClient::connect( const std::string & hostname,
 /*!
 
 */
-void
-DebugClient::setWriteMode( const bool on )
+bool
+DebugClient::open( const std::string & log_dir,
+                   const std::string & teamname,
+                   const int unum )
 {
-    if ( on )
+    if ( M_server_log.is_open() )
     {
-        M_on = true;
-        M_write_mode = true;
+        M_server_log.close();
     }
+
+    std::ostringstream filepath;
+    if ( ! log_dir.empty() )
+    {
+        filepath << log_dir;
+        if ( *log_dir.rbegin() != '/' )
+        {
+            filepath << '/';
+        }
+    }
+    filepath << teamname << '-' << unum << ".dcl";
+
+    M_server_log.open( filepath.str().c_str() );
+
+    if ( ! M_server_log.is_open() )
+    {
+        return false;
+    }
+
+    M_on = true;
+    M_write_mode = true;
+    return true;
 }
 
 /*-------------------------------------------------------------------*/
@@ -272,12 +321,12 @@ DebugClient::setWriteMode( const bool on )
 
 */
 void
-DebugClient::writeAll( const WorldModel & world )
+DebugClient::writeAll( const WorldModel & world,
+                       const ActionEffector & effector )
 {
     if ( M_on )
     {
-        this->toStr( world );
-
+        this->toStr( world, effector );
         if ( M_connected )
         {
             this->send();
@@ -307,6 +356,12 @@ DebugClient::close()
         M_socket.reset();
     }
 
+    if ( M_server_log.is_open() )
+    {
+        M_server_log.flush();
+        M_server_log.close();
+    }
+
     M_write_mode = false;
 
     M_on = false;
@@ -317,11 +372,12 @@ DebugClient::close()
 
 */
 void
-DebugClient::toStr( const WorldModel & world )
+DebugClient::toStr( const WorldModel & world,
+                    const ActionEffector & effector )
 {
     std::ostringstream ostr;
 
-    ostr << "((debug (format-version 2)) (time "
+    ostr << "((debug (format-version 3)) (time "
          << world.time().cycle() << ")";
 
 
@@ -405,8 +461,29 @@ DebugClient::toStr( const WorldModel & world )
                    world.unknownPlayers().end(),
                    PlayerPrinter( ostr, 'u' ) );
 
+    // say message
+    if ( ! effector.getSayMessage().empty() )
+    {
+        ostr << " (say \"{" << effector.getSayMessage() << "} ";
+        for ( std::vector< const SayMessage * >::const_iterator it = effector.sayMessageCont().begin();
+              it != effector.sayMessageCont().end();
+              ++it )
+        {
+            (*it)->printDebug( ostr );
+        }
+        ostr << "\")";
+    }
+
+    // heard information
+    if ( world.audioMemory().time() == world.time() )
+    {
+        ostr << " (hear ";
+        world.audioMemory().printDebug( ostr );
+        ostr << ')';
+    }
+
     // target number
-    if ( M_target_unum != 0 )
+    if ( M_target_unum != Unum_Unknown )
     {
         ostr << " (target-teammate " << M_target_unum << ")";
     }
@@ -468,14 +545,26 @@ DebugClient::send()
 void
 DebugClient::write( const long & cycle )
 {
+#if 0
     if ( dlog.isOpen() )
+    {
+//         char buf[32];
+//         snprintf( buf, 32, "%%%% step %ld\n", cycle );
+//         dlog.print( buf );
+//         dlog.print( "%% debug [" );
+//         dlog.print( M_main_buffer.c_str() );
+//         dlog.print( "]\n" );
+        dlog.printDebugClientMessage( cycle, M_main_buffer );
+    }
+#endif
+
+    if ( M_server_log.is_open() )
     {
         char buf[32];
-        std::snprintf( buf, 32, "%%%% step %ld\n", cycle );
-        dlog.print( buf );
-        dlog.print( "%% debug [" );
-        dlog.print( M_main_buffer.c_str() );
-        dlog.print( "]\n" );
+        snprintf( buf, 32, "%%%% step %ld\n", cycle );
+
+        M_server_log << buf << "%% debug [" << M_main_buffer << "]"
+                     << std::endl;
     }
 }
 
@@ -487,9 +576,11 @@ void
 DebugClient::clear()
 {
     M_main_buffer.erase();
-    M_target_unum = 0;
+
+    M_target_unum = Unum_Unknown;
     M_target_point.invalidate();
     M_message.erase();
+
     M_lines.clear();
     M_triangles.clear();
     M_rectangles.clear();
@@ -501,7 +592,8 @@ DebugClient::clear()
 
 */
 void
-DebugClient::addMessage( char * msg, ... )
+DebugClient::addMessage( const char * msg,
+                         ... )
 {
     if ( M_on )
     {
@@ -527,7 +619,7 @@ DebugClient::addLine( const Vector2D & from,
     {
         if ( M_lines.size() < MAX_LINE )
         {
-            M_lines.push_back( std::make_pair( from, to ) );
+            M_lines.push_back( Segment2D( from, to ) );
         }
     }
 }
