@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -34,6 +34,7 @@
 
 #include <rcsc/player/visual_sensor.h>
 #include <rcsc/geom/vector_2d.h>
+#include <rcsc/game_time.h>
 #include <rcsc/types.h>
 
 #include <boost/scoped_ptr.hpp>
@@ -157,22 +158,38 @@ public:
     /*!
       \brief implicitly delete internal impl
     */
+    virtual
     ~Localization();
 
 public:
+
     /*!
-      \brief localize self.
+      \brief estimate self facing direction.
+      \param see analyzed see information
+      \param self_face pointer to the result variable
+      \param self_face_err pointer to the result variable
+      \return result
+     */
+    virtual
+    bool estimateSelfFace( const VisualSensor & see,
+                           double * self_face,
+                           double * self_face_err );
+
+    /*!
+      \brief localize self position.
       \param see analyzed see info
-      \param self_face pointer to the variable to store the localized face angle
-      \param self_face_err pointer to the variable to store the localized face angle error
+      \param self_face localized face angle
+      \param self_face_err localized face angle error
       \param self_pos pointer to the variable to store the localized self position
       \param self_pos_err pointer to the variable to store the localized self position error
+      \return if failed, returns false
     */
-    void localizeSelf( const VisualSensor & see,
-                       double * self_face,
-                       double * self_face_err,
+    virtual
+    bool localizeSelf( const VisualSensor & see,
+                       const double & self_face,
+                       const double & self_face_err,
                        Vector2D * self_pos,
-                       Vector2D * self_pos_err ) const;
+                       Vector2D * self_pos_err );
 
     /*!
       \brief localze ball relative info
@@ -183,14 +200,16 @@ public:
       \param rpos_err pointer to the variable to store the localized relative position error
       \param rvel pointer to the variable to store the localized relative velocity
       \param rvel_err pointer to the variable to store the localized relative velocity error
+      \return if failed, returns false
     */
-    void localizeBallRelative( const VisualSensor & see,
+    virtual
+    bool localizeBallRelative( const VisualSensor & see,
                                const double & self_face,
                                const double & self_face_err,
                                Vector2D * rpos,
                                Vector2D * rpos_err,
                                Vector2D * rvel,
-                               Vector2D * rvel_err ) const;
+                               Vector2D * rvel_err );
 
     /*!
       \brief localze other player
@@ -200,13 +219,15 @@ public:
       \param self_pos localized self position
       \param self_vel localized self velocity
       \param to pointer to the variable to store the localized result
+      \return if failed, returns false
     */
-    void localizePlayer( const VisualSensor::PlayerT & from,
+    virtual
+    bool localizePlayer( const VisualSensor::PlayerT & from,
                          const double & self_face,
                          const double & self_face_err,
                          const Vector2D & self_pos,
                          const Vector2D & self_vel,
-                         PlayerT * to ) const;
+                         PlayerT * to );
 };
 
 }
