@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -148,7 +148,7 @@ SeeState::updateBySenseBody( const GameTime & sense_time,
     if ( M_view_width != vw )
     {
         dlog.addText( Logger::SYSTEM,
-                      "SeeState. updateBySenseBody. view_width does not match. old=%d sense=%d",
+                      __FILE__" (updateBySenseBody) view_width does not match. old=%d sense=%d",
                       M_view_width.type(), vw.type() );
         std::cerr << sense_time
                   << " view width does not match . old=" << M_view_width.type()
@@ -161,7 +161,7 @@ SeeState::updateBySenseBody( const GameTime & sense_time,
     if ( M_view_quality != vq )
     {
         dlog.addText( Logger::SYSTEM,
-                      "SeeState. updateBySenseBody. view_quality does not match. old=%d sense=%d",
+                      __FILE__" (updateBySenseBody) view_quality does not match. old=%d sense=%d",
                       M_view_quality.type(), vq.type() );
         std::cerr << sense_time
                   << " view quality does not match. old=" << M_view_quality.type()
@@ -188,7 +188,7 @@ SeeState::updateBySee( const GameTime & see_time,
         if ( isSynch() )
         {
             dlog.addText( Logger::SYSTEM,
-                          "SeeState.updateSee. estimated synch, but duplicated" );
+                          __FILE__" (updateBySee) estimated synch, but duplicated" );
             std::cerr << see_time
                       << " SeeState.updateSee. estimated synch, but duplicated"
                       << std::endl;
@@ -212,7 +212,7 @@ SeeState::updateBySee( const GameTime & see_time,
     if ( ! isSynch() )
     {
         dlog.addText( Logger::SYSTEM,
-                      "SeeState.updateSee. but no synch" );
+                      __FILE__" (updateBySee) but no synch" );
         return;
     }
 
@@ -235,7 +235,7 @@ SeeState::updateBySee( const GameTime & see_time,
                   << std::endl;
     }
     dlog.addText( Logger::SYSTEM,
-                  "SeeState. see update, prev timing = %d.  current timing = %d",
+                  __FILE__" (updateBySee) see update, prev timing = %d.  current timing = %d",
                   M_last_timing, new_timing );
 
     M_last_timing = new_timing;
@@ -268,7 +268,7 @@ SeeState::setNewCycle( const GameTime & new_time )
     M_current_see_count = 0;
 
     dlog.addText( Logger::SYSTEM,
-                  "SeeState. set new cycle. cycle till next see = %d",
+                  __FILE__" (updateBySee) set new cycle. cycle till next see = %d",
                   M_cycles_till_next_see );
 }
 
@@ -280,7 +280,7 @@ void
 SeeState::setLastSeeTiming( const Timing last_timing )
 {
     dlog.addText( Logger::SYSTEM,
-                  "SeeState.setTiming %d.",
+                  __FILE__" (setLastSeeTiming) %d",
                   last_timing );
     M_last_timing = last_timing;
 }
@@ -411,7 +411,7 @@ SeeState::setViewMode( const ViewWidth & new_width,
     if ( M_last_see_time != M_current_time )
     {
         dlog.addText( Logger::SYSTEM,
-                      "SeeState. setViewMode. no current cycle see arrival" );
+                      __FILE__" (setViewMode) no current cycle see arrival" );
         return;
     }
 
@@ -425,19 +425,19 @@ SeeState::setViewMode( const ViewWidth & new_width,
             M_cycles_till_next_see = 3;
             M_synch_type = SYNCH_WIDE;
             dlog.addText( Logger::SYSTEM,
-                          "SeeState. setViewMode. synch wide: cycle = 3" );
+                          __FILE__" (setViewMode) synch wide: cycle = 3" );
             break;
         case ViewWidth::NORMAL:
             M_cycles_till_next_see = 2;
             M_synch_type = SYNCH_NORMAL;
             dlog.addText( Logger::SYSTEM,
-                          "SeeState. setViewMode. synch normal: cycle = 2" );
+                          __FILE__" (setViewMode) synch normal: cycle = 2" );
             break;
         case ViewWidth::NARROW:
             M_cycles_till_next_see = 1;
             M_synch_type = SYNCH_NARROW;
             dlog.addText( Logger::SYSTEM,
-                          "SeeState. setViewMode. synch narrow: cycle = 1" );
+                          __FILE__" (setViewMode) synch narrow: cycle = 1" );
             break;
         default:
             break;
@@ -454,14 +454,14 @@ SeeState::setViewMode( const ViewWidth & new_width,
             M_cycles_till_next_see = 3;
             M_synch_type = SYNCH_WIDE;
             dlog.addText( Logger::SYSTEM,
-                          "SeeState. setViewMode. 00:wide: cycle = %d",
+                          __FILE__" (setViewMode) 00:wide: cycle = %d",
                           M_cycles_till_next_see );
             break;
         case ViewWidth::NORMAL:
             M_cycles_till_next_see = 1;
             M_synch_type = SYNCH_EVERY;
             dlog.addText( Logger::SYSTEM,
-                          "SeeState. setViewMode. 00:normal: cycle = %d",
+                          __FILE__" (setViewMode) 00:normal: cycle = %d",
                           M_cycles_till_next_see );
             break;
         case ViewWidth::NARROW:
@@ -485,21 +485,21 @@ SeeState::setViewMode( const ViewWidth & new_width,
             M_cycles_till_next_see = 3;
             M_synch_type = SYNCH_WIDE;
             dlog.addText( Logger::SYSTEM,
-                          "SeeState. setViewMode. 50:wide: cycle = %d",
+                          __FILE__" (setViewMode) 50:wide: cycle = %d",
                           M_cycles_till_next_see );
             break;
         case ViewWidth::NORMAL:
             M_cycles_till_next_see = 2;
             M_synch_type = SYNCH_NORMAL;
             dlog.addText( Logger::SYSTEM,
-                          "SeeState. setViewMode. 50:normal: cycle = %d",
+                          __FILE__" (setViewMode) 50:normal: cycle = %d",
                           M_cycles_till_next_see );
             break;
         case ViewWidth::NARROW:
             M_cycles_till_next_see = 1;
             M_synch_type = SYNCH_EVERY;
             dlog.addText( Logger::SYSTEM,
-                          "SeeState. setViewMode. 50:narrow: cycle = %d",
+                          __FILE__" (setViewMode) 50:narrow: cycle = %d",
                           M_cycles_till_next_see );
             break;
         default:
