@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -49,6 +49,8 @@
 #include <iostream>
 #include <cmath>
 
+//#define DEBUG_PRINT
+
 namespace rcsc {
 
 int SelfObject::S_pos_count_thr = 20;
@@ -91,6 +93,8 @@ SelfObject::SelfObject()
     M_type = Hetero_Default;
     M_player_type = PlayerTypeSet::i().get( Hetero_Default );
 
+    M_dist_from_self = 0.0;
+
     for ( int i = 0; i < 4; ++i )
     {
         M_last_moves[i].assign( 0.0, 0.0 );
@@ -128,6 +132,18 @@ SelfObject::init( const SideID side,
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+void
+SelfObject::setPlayerType( const int id )
+{
+    AbstractPlayerObject::setPlayerType( id );
+
+    M_stamina.init( playerType() );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 bool
 SelfObject::posValid() const
@@ -177,8 +193,8 @@ SelfObject::catchableArea() const
     //double t = atan2(0.5, 2.0); // atan2(w * 0.5, l);
     //double len = 2.0 / cos(t);
 
-    return std::sqrt( std::pow( ServerParam::i().catchAreaWidth() * 0.5, 2.0 )
-                      + std::pow( ServerParam::i().catchAreaLength(), 2.0 ) );
+    return std::sqrt( std::pow( ServerParam::i().catchAreaWidth() * 0.5, 2 )
+                      + std::pow( ServerParam::i().catchAreaLength(), 2 ) );
 }
 
 /*-------------------------------------------------------------------*/
@@ -257,6 +273,14 @@ SelfObject::update( const ActionEffector & act,
     // stamina
     M_stamina.update( playerType(), dash_power );
 
+#ifdef DEBUG_PRINT
+    dlog.addText( Logger::WORLD,
+                  __FILE__" (update) estimated stamina=%.1f effort=%f recovery=%f capacity=%.1f",
+                  M_stamina.stamina(),
+                  M_stamina.effort(),
+                  M_stamina.recovery(),
+                  M_stamina.staminaCapacity() );
+#endif
 
     /////////////////////////////////////////////////////////////////
     // turn
@@ -307,11 +331,13 @@ SelfObject::update( const ActionEffector & act,
 
         M_vel_error.add( velrnd, velrnd ); // add new vel rand
 
+#ifdef DEBUG_PRINT
         dlog.addText( Logger::WORLD,
-                      __FILE__": update. pos=(%.2f, %.2f) pos_err(%.3f, %.3f) "
+                      __FILE__" (update) pos=(%.2f, %.2f) pos_err(%.3f, %.3f) "
                       "vel=(%.2f, %.2f) vel_err=(%.3f, %.3f)",
                       M_pos.x, M_pos.y, M_pos_error.x, M_pos_error.y,
                       M_vel.x, M_vel.y, M_vel_error.x, M_vel_error.y );
+#endif
     }
 
     /////////////////////////////////////////////////////////////////
@@ -459,7 +485,7 @@ SelfObject::updateAfterSenseBody( const BodySensor & sense,
 
     ////////////////////////////////////////////////////
     // stamina
-    M_stamina.updateAfterSense( playerType(), sense, current );
+    M_stamina.updateAfterSense( sense, current );
 
     ////////////////////////////////////////////////////
     // neck relative
@@ -481,29 +507,40 @@ SelfObject::updateAfterSenseBody( const BodySensor & sense,
         if ( sense.noneCollided() )
         {
             M_collides_with_none = true;
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::WORLD,
-                          __FILE__": self.updateAfterSense. no collision" );
+                          __FILE__" (updateAfterSense) no collision" );
+#endif
         }
 
         if ( sense.ballCollided() )
         {
+            M_collision_estimated = true;
             M_collides_with_ball = true;
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::WORLD,
-                          __FILE__": self.updateAfterSense. collided with ball" );
+                          __FILE__" (updateAfterSense) collided with ball" );
+#endif
         }
 
         if ( sense.playerCollided() )
         {
+            M_collision_estimated = true;
             M_collides_with_player = true;
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::WORLD,
-                          __FILE__": self.updateAfterSense. collided with player" );
+                          __FILE__" (updateAfterSense) collided with player" );
+#endif
         }
 
         if ( sense.postCollided() )
         {
+            M_collision_estimated = true;
             M_collides_with_post = true;
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::WORLD,
-                          __FILE__": self.updateAfterSense. collided with post" );
+                          __FILE__" (updateAfterSense) collided with post" );
+#endif
         }
     }
 
@@ -552,11 +589,15 @@ SelfObject::updateAfterSenseBody( const BodySensor & sense,
         // sensed speed is rounded by 0.01
         M_vel_error.assign( ( maxcos - mincos ) * ( sense.speedMag() + 0.005 ),
                             ( maxsin - minsin ) * ( sense.speedMag() + 0.005 ) );
+
+#ifdef DEBUG_PRINT
         dlog.addText( Logger::WORLD,
-                      __FILE__": updateAfterSense. vel=(%.2f %.2f) vel_err=(%.3f, %.3f)  faceErr = %.3f",
+                      __FILE__" (updateAfterSense)"
+                      " vel=(%.2f %.2f) vel_err=(%.3f, %.3f)  faceErr = %.3f",
                       M_vel.x, M_vel.y,
                       M_vel_error.x, M_vel_error.y,
                       faceError() );
+#endif
 
         ////////////////////////////////////////////////////
         // collision check
@@ -567,9 +608,12 @@ SelfObject::updateAfterSenseBody( const BodySensor & sense,
                                      * 0.11 )
              )
         {
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::WORLD,
-                          __FILE__": updateAfterSense. check collisiion. estimate_vel=(%.2f %.2f)",
+                          __FILE__" (updateAfterSense)"
+                          " check collisiion. estimate_vel=(%.2f %.2f)",
                           estimate_vel.x, estimate_vel.y );
+#endif
             // decay is very small
             //   default decay = 0.4
             //   collision decay = 0.1
@@ -583,8 +627,11 @@ SelfObject::updateAfterSenseBody( const BodySensor & sense,
                  )
             {
                 M_collision_estimated = true;
+#ifdef DEBUG_PRINT
                 dlog.addText( Logger::WORLD,
-                              __FILE__": self.updateAfterSense. *COLLISION* estimated" );
+                              __FILE__" (updateAfterSense)"
+                              " self.updateAfterSense. *COLLISION* estimated" );
+#endif
             }
         }
 
@@ -594,43 +641,36 @@ SelfObject::updateAfterSenseBody( const BodySensor & sense,
 
         //////////////////////////////////////////////////////
         // pointto
-        if ( sense.armExpires() != 0
-             && posValid() )
-        {
-            M_pointto_pos
-                = M_pos
-                + Vector2D::polar2vector( sense.pointtoDist(),
-                                          face() + sense.pointtoDir() );
-        }
-        else
+        if ( sense.armExpires() == 0 )
         {
             M_pointto_pos.invalidate();
         }
 
         ////////////////////////////////////////////////////
         // last move
-        if ( ! M_collision_estimated )
+        if ( ! collisionEstimated() )
         {
             M_last_move = M_vel / this->playerType().playerDecay();
         }
-        else if ( velValid() )
-        {
-            // use estimated vel !!!
-            M_last_move = estimate_vel / this->playerType().playerDecay();
-        }
         else
         {
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::WORLD,
-                          __FILE__": updateAfterSense. XXX reset last move" );
+                          __FILE__" (updateAfterSense) XXX reset last move" );
+#endif
             M_last_move.invalidate();
         }
+
         M_last_moves[0] = M_last_move;
 
         if ( collisionEstimated()
              || collidesWithBall() )
         {
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::WORLD,
-                          __FILE__": updateAfterSense. collision. reset last move" );
+                          __FILE__" (updateAfterSense)"
+                          " collision. reset last move" );
+#endif
             M_last_moves[0].invalidate();
         }
     }
@@ -642,18 +682,24 @@ SelfObject::updateAfterSenseBody( const BodySensor & sense,
     // tackle
     if ( M_tackle_expires > 0 )
     {
+#ifdef DEBUG_PRINT
         dlog.addText( Logger::WORLD,
-                      __FILE__": updateAfterSense internal_tackle_exp=%d, sense_body=%d",
+                      __FILE__" (updateAfterSense)"
+                      " internal_tackle_exp=%d, sense_body=%d",
                       M_tackle_expires, sense.tackleExpires() );
+#endif
     }
 
     if ( M_tackle_expires != sense.tackleExpires() )
     {
         std::cerr << current
-                  << "self : updateAfterSense. internal tackle expire has invalie value."
+                  << " (updateAfterSense) internal tackle expire has invalie value."
                   << std::endl;
+#ifdef DEBUG_PRINT
         dlog.addText( Logger::WORLD,
-                      __FILE__": updateAfterSense internal_tackle_exp is not match to sense_body" );
+                      __FILE__" (updateAfterSense)"
+                      " internal_tackle_exp is not match to sense_body" );
+#endif
     }
 
     M_tackle_expires = sense.tackleExpires();
@@ -692,12 +738,6 @@ SelfObject::updateAfterFullstate( const FullstateSensor::PlayerT & my_state,
     M_face_count = 0;
 
     M_stamina.updateAfterFullstate( my_state );
-
-    M_pointto_pos
-        = pos()
-        + Vector2D::polar2vector( my_state.pointto_dist_,
-                                  face() + my_state.pointto_dir_ );
-
 }
 
 
@@ -710,7 +750,7 @@ SelfObject::updatePosBySee( const Vector2D & pos,
                             const Vector2D & pos_err,
                             const double & face,
                             const double & face_err,
-                            const GameTime & current)
+                            const GameTime & current )
 {
     // other param is updated in update or update_after_sense
     M_time = current;
@@ -720,6 +760,7 @@ SelfObject::updatePosBySee( const Vector2D & pos,
     {
         Vector2D new_pos = pos;
         Vector2D new_error = pos_err;
+
         // and last error is smaller than now
         if ( M_pos_error.x < pos_err.x )
         {
@@ -727,28 +768,47 @@ SelfObject::updatePosBySee( const Vector2D & pos,
                 = pos.x
                 + ( M_pos.x - pos.x ) * ( pos_err.x / ( M_pos_error.x + pos_err.x ) );
             new_error.x = ( M_pos_error.x + pos_err.x ) * 0.5;
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::WORLD,
-                          __FILE__": updatePosBySee. X modified [%f] < [%f]",
+                          __FILE__" (updatePosBySee)"
+                          " X modified [%f] < [%f]",
                           M_pos_error.x, pos_err.x );
+#endif
         }
+
         if  ( M_pos_error.y < pos_err.y )
         {
             new_pos.y
                 = pos.y
                 + ( M_pos.y - pos.y ) * ( pos_err.y / ( M_pos_error.y + pos_err.y ) );
             new_error.y = ( M_pos_error.y + pos_err.y ) * 0.5;
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::WORLD,
-                          __FILE__": updatePosBySee. Y modified [%f] < [%f]",
+                          __FILE__" (updatePosBySee)"
+                          " Y modified [%f] < [%f]",
                           M_pos_error.y, pos_err.y );
+#endif
         }
+
         M_pos = new_pos;
         M_pos_error = new_error;
 
+#ifdef DEBUG_PRINT
         dlog.addText( Logger::WORLD,
-                      __FILE__": updatePosBySee. seen_pos=(%.3f, %.3f) average_pos=(%.3f, %.3f) "
+                      __FILE__" (updatePosBySee)"
+                      " seen_pos=(%.3f, %.3f) average_pos=(%.3f, %.3f) "
                       " error=(%.4f, %.4f)",
                       pos.x, pos.y, new_pos.x, new_pos.y,
                       M_pos_error.x, M_pos_error.y );
+#endif
+
+        if ( M_seen_pos_count == 1
+             && ( collisionEstimated()
+                  || M_last_move.valid() ) )
+        {
+            M_last_move = new_pos - M_seen_pos;
+            M_last_moves[0] = M_last_move;
+        }
     }
     else
     {
@@ -852,17 +912,29 @@ SelfObject::updateVelDirAfterSee( const BodySensor & sense,
         M_vel_error.assign( ( maxcos - mincos ) * ( sense.speedMag() + 0.005 ),
                             ( maxsin - minsin ) * ( sense.speedMag() + 0.005 ) );
 
-        // set last move
-        M_last_move = M_vel / this->playerType().playerDecay();
-        M_last_moves[0] = M_last_move;
-
+#ifdef DEBUG_PRINT
         dlog.addText( Logger::WORLD,
-                      __FILE__": updateVelDirAfterSee. face_error=%.2f, sensed_dir_error=%.2f."
-                      " vel=(%f %f) vel_err=(%f %f) last_move=(%.4f %.4f)",
+                      __FILE__" (updateVelDirAfterSee)"
+                      " face_error=%.2f, sensed_dir_error=%.2f."
+                      " vel=(%f %f) vel_err=(%f %f)",
                       faceError(), sensed_speed_dir_error,
                       M_vel.x, M_vel.y,
-                      M_vel_error.x, M_vel_error.y,
-                      M_last_move.x, M_last_move.y );
+                      M_vel_error.x, M_vel_error.y );
+#endif
+
+        // set last move
+        if ( ! collisionEstimated() )
+        {
+            M_last_move = M_vel / this->playerType().playerDecay();
+            M_last_moves[0] = M_last_move;
+
+#ifdef DEBUG_PRINT
+            dlog.addText( Logger::WORLD,
+                          __FILE__" (updateVelDirAfterSee)"
+                          " last_move=(%.4f %.4f)",
+                          M_last_move.x, M_last_move.y );
+#endif
+        }
     }
 }
 
@@ -883,13 +955,21 @@ SelfObject::updateBallInfo( const BallObject & ball )
     }
 
     M_dist_from_ball = ball.distFromSelf();
+    M_angle_from_ball = ball.angleFromSelf() + AngleDeg( 180.0 );
 
     //------------------------------------------------------
     // check kickable
 
-    if ( ball.distFromSelf() <= this->kickableArea() - 0.055 )
+    if ( ball.distFromSelf() <= this->kickableArea() )
     {
-        M_kickable = true;
+        double buf = 0.055;
+        if ( ball.seenPosCount() >= 2 ) buf = 0.255;
+        if ( ball.seenPosCount() >= 1 ) buf = 0.155;
+
+        if ( ball.distFromSelf() <= this->kickableArea() - buf )
+        {
+            M_kickable = true;
+        }
 
         M_kick_rate
             = kick_rate( ball.distFromSelf(),
@@ -933,9 +1013,37 @@ SelfObject::updateBallInfo( const BallObject & ball )
         M_tackle_probability = 1.0 - fail_prob;
     }
 
+#ifdef DEBUG_PRINT
     dlog.addText( Logger::WORLD,
-                  __FILE__": updateBallInfo. ball_dist=%.3f.  my_kickable=%.3f.  tackle_prob=%.3f",
+                  __FILE__" (updateBallInfo) ball_dist=%.3f my_kickable=%.3f tackle_prob=%.3f",
                   ball.distFromSelf(), kickableArea(), M_tackle_probability );
+#endif
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+SelfObject::updateKickableState( const BallObject & ball,
+                                 const int self_reach_cycle,
+                                 const int teammate_reach_cycle,
+                                 const int opponent_reach_cycle )
+{
+    if ( ! M_kickable
+         && ball.seenPosCount() == 0
+         && ( self_reach_cycle >= 10 // XXX magic number XXX
+              || ( opponent_reach_cycle < self_reach_cycle - 7
+                   && opponent_reach_cycle < teammate_reach_cycle - 7 ) )
+         && ball.distFromSelf() < this->kickableArea() - 0.001 )
+    {
+        M_kickable = true;
+
+#ifdef DEBUG_PRINT
+        dlog.addText( Logger::WORLD,
+                      __FILE__" (updateKickableState) set to kickable" );
+#endif
+    }
 }
 
 /*-------------------------------------------------------------------*/
@@ -958,11 +1066,13 @@ SelfObject::getSafetyDashPower( const double & dash_power ) const
                   << " input dash power = " << dash_power
                   << " normalized power=" << normalized_power
                   << std::endl;
+#ifdef DEBUG_PRINT
         dlog.addText( Logger::ACTION,
-                      __FILE__": getSafetyDashPower. input_dash_power=%.2f normalized_power=%.1f required_stamina=%.1f",
+                      __FILE__" (getSafetyDashPower) input_dash_power=%.2f normalized_power=%.1f required_stamina=%.1f",
                       dash_power,
                       normalized_power,
                       required_stamina );
+#endif
     }
 
     // check rest stamina
@@ -984,9 +1094,9 @@ SelfObject::getSafetyDashPower( const double & dash_power ) const
         return normalized_power;
     }
 
-    dlog.addText( Logger::ACTION,
-                  __FILE__": getSafetyDashPower. cur_stamina=%.1f, required=%.1f, result=%.2f",
-                  stamina(), required_stamina, result_power );
+    //dlog.addText( Logger::ACTION,
+    //              __FILE__" (getSafetyDashPower) cur_stamina=%.1f, required=%.1f, result=%.2f",
+    //              stamina(), required_stamina, result_power );
     return result_power;
 }
 
