@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -37,7 +37,6 @@
 #include <rcsc/player/stamina_model.h>
 #include <rcsc/player/fullstate_sensor.h>
 
-#include <rcsc/common/player_type.h>
 #include <rcsc/geom/vector_2d.h>
 #include <rcsc/game_time.h>
 #include <rcsc/types.h>
@@ -86,6 +85,7 @@ private:
     int M_tackle_expires;
 
     int M_arm_movable; //!< cycles till arm movable
+    Vector2D M_pointto_rpos; //!< estimated pointing position relative to the neck
     Vector2D M_pointto_pos; //!< estimated pointing position
     GameTime M_last_pointto_time; //!< action performed time
 
@@ -158,6 +158,13 @@ public:
           return true;
       }
 
+    /*!
+      \brief update player type id
+      \param type new player type id
+     */
+    virtual
+    void setPlayerType( const int type );
+
     /*!
       \brief get player type parameter
       \return const reference to the player type object
@@ -294,6 +301,15 @@ public:
           return M_tackle_expires > 0;
       }
 
+    /*!
+      \brief check if player is tackling or not.
+      \return checked result.
+     */
+    bool isTackling() const
+      {
+          return isFreezed();
+      }
+
     /*!
       \brief get arm movable count
       \return cycles till arm is movable
@@ -341,6 +357,16 @@ public:
           return M_attentionto_unum;
       }
 
+    /*!
+      \brief get current stamina model.
+      \return stamina model object.
+     */
+    const
+    StaminaModel & staminaModel() const
+      {
+          return M_stamina;
+      }
+
     /*!
       \brief get current stamina value
       \return raw stamina value from stamina object
@@ -416,7 +442,7 @@ public:
     */
     bool collidesWithPlayer() const
       {
-          return M_collides_with_ball;
+          return M_collides_with_player;
       }
 
     /*!
@@ -462,8 +488,7 @@ public:
       \brief get current estimated kick power rate
       \return calculated kick rate value
     */
-    const
-    double & kickRate() const
+    double kickRate() const
       {
           return M_kick_rate;
       }
@@ -524,9 +549,9 @@ public:
       \param act internal action info
       \param current current game time
     */
-    void updateAfterSense( const BodySensor & sense,
-                           const ActionEffector & act,
-                           const GameTime & current );
+    void updateAfterSenseBody( const BodySensor & sense,
+                               const ActionEffector & act,
+                               const GameTime & current );
 
     /*!
       \brief update status using fullstate info
@@ -546,11 +571,11 @@ public:
       \param face_err estimated error of facing angle
       \param current current game time
     */
-    void updatePos( const Vector2D & pos,
-                    const Vector2D & pos_err,
-                    const double & face,
-                    const double & face_err,
-                    const GameTime & current );
+    void updatePosBySee( const Vector2D & pos,
+                         const Vector2D & pos_err,
+                         const double & face,
+                         const double & face_err,
+                         const GameTime & current );
 
     /*!
       \brief apply collision effect
@@ -566,9 +591,9 @@ public:
       \param face_err estimated error of facing angle
       \param current current game time
     */
-    void updateAngle( const double & face,
-                      const double & face_err,
-                      const GameTime & current );
+    void updateAngleBySee( const double & face,
+                           const double & face_err,
+                           const GameTime & current );
 
     /*!
       \brief adjust velocity direction using observed facing angle and sensed speed
@@ -589,6 +614,20 @@ public:
     */
     void updateBallInfo( const BallObject & ball );
 
+    /*!
+      \brief update ball kickable state
+      \param ball ball object
+      \param self_reach_cycle estimated ball interception step.
+      \param teammate_reach_cycle estimated ball interception step.
+      \param opponent_reach_cycle estimated ball interception step.
+
+      This method is called just before action decision
+    */
+    void updateKickableState( const BallObject & ball,
+                              const int self_reach_cycle,
+                              const int teammate_reach_cycle,
+                              const int opponent_reach_cycle );
+
     /////////////////////////////////////////////////////////////
     // used for direct update
 
@@ -651,26 +690,6 @@ public:
      */
     bool canTurn( const double & moment ) const;
 
-
-    /*!
-      \brief estimate reach point
-      \param n_step this method estimates ball point after this steps
-      \return estimated point vector
-    */
-    Vector2D inertiaPoint( const int n_step ) const
-      {
-          return playerType().inertiaPoint( pos(), vel(), n_step );
-      }
-
-    /*!
-      \brief estimate final reach point
-      \return estimated point vector
-    */
-    Vector2D inertiaFinalPoint() const
-      {
-          return playerType().inertiaFinalPoint( pos(), vel() );
-      }
-
 };
 
 }
