@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -270,27 +270,27 @@ private:
 public:
     /*!
       \brief update by sense_body.
-      \param sense sense_body info
+      \param sense_body analyzed sense_body info
       \param act action effector
       \param current current game time
 
       This method is called just after sense_body message receive
     */
-    void updateAfterSenseBody( const BodySensor & sense,
+    void updateAfterSenseBody( const BodySensor & sense_body,
                                const ActionEffector & act,
                                const GameTime & current );
 
     /*!
       \brief update by see info
       \param see analyzed see info
-      \param sense analyzed sense_body info
+      \param sense_body analyzed sense_body info
       \param act action effector
       \param current current game time
 
       This method is called just after see message receive
     */
     void updateAfterSee( const VisualSensor & see,
-                         const BodySensor & sense,
+                         const BodySensor & sense_body,
                          const ActionEffector & act,
                          const GameTime & current );
 
@@ -349,9 +349,12 @@ private:
     /*!
       \brief self localization
       \param see analyzed see info
+      \param sense_body analyzed sense_body info
       \param current current game time
+      \return if failed, returns false
     */
-    void localizeSelf( const VisualSensor & see,
+    bool localizeSelf( const VisualSensor & see,
+                       const BodySensor & sense_body,
                        const GameTime & current );
 
     /*!
@@ -368,16 +371,14 @@ private:
       \brief estimate ball velocity using position difference
       \param see analyzed see info
       \param act action effector
-      \param current current game time
-      \param current seen relative pos
-      \param current seen relative pos error
+      \param rpos seen relative pos
+      \param rpos_error seen relative pos error
       \param vel reference to the velocity variable
       \param vel_error reference to the velocity error variable
       \param vel_count reference to the velocity count variable
     */
     void estimateBallVelByPosDiff( const VisualSensor & see,
                                    const ActionEffector & act,
-                                   const GameTime & current,
                                    const Vector2D & rpos,
                                    const Vector2D & rpos_error,
                                    Vector2D & vel,
@@ -479,6 +480,10 @@ private:
     */
     void updateDefenseLine();
 
+    /*!
+      \brief update back passer candidates.
+     */
+    void updateBackPasser();
 public:
 
     /*!
@@ -642,7 +647,7 @@ public:
       }
 
     /*!
-      \brief get opponent info
+      \brief get opponent info (except unknown players)
       \return const reference to the PlayerObject container
     */
     const
@@ -674,7 +679,7 @@ public:
       }
 
     /*!
-      \brief get opponents sorted by distance from self
+      \brief get opponents sorted by distance from self (includes unknown players)
       \return const reference to the PlayerObject pointer container
     */
     const
@@ -694,7 +699,7 @@ public:
       }
 
     /*!
-      \brief get opponents sorted by distance from ball
+      \brief get opponents sorted by distance from ball (includes unknown players)
       \return const reference to the PlayerObject pointer container
     */
     const
@@ -722,7 +727,7 @@ public:
       }
 
     /*!
-      \brief get all players including self.
+      \brief get all players includes self.
       \return const rerefence to the AbstractPlayerObject pointer container..
      */
     const
@@ -732,7 +737,7 @@ public:
       }
 
     /*!
-      \brief get all teammates including self.
+      \brief get all teammate playerss includes self.
       \return const rerefence to the AbstractPlayerObject pointer container..
      */
     const
@@ -742,7 +747,7 @@ public:
       }
 
     /*!
-      \brief get all opponents including self.
+      \brief get all opponent players (includes unknown players)
       \return const rerefence to the AbstractPlayerObject pointer container..
      */
     const
@@ -758,21 +763,6 @@ public:
       \param unum wanted player's uniform number
       \return const pointer to the PlayerObject instance or NULL
     */
-//     const
-//     PlayerObject * getTeammate( const int unum ) const
-//       {
-//           const PlayerPtrCont::const_iterator end = teammatesFromSelf().end();
-//           for ( PlayerPtrCont::const_iterator it = teammatesFromSelf().begin();
-//                 it != end;
-//                 ++it )
-//           {
-//               if ( (*it)->unum() == unum )
-//               {
-//                   return *it;
-//               }
-//           }
-//           return static_cast< PlayerObject * >( 0 );
-//       }
     const
     AbstractPlayerObject * teammate( const int unum ) const
       {
@@ -785,21 +775,6 @@ public:
       \param unum wanted player's uniform number
       \return const pointer to the PlayerObject instance or NULL
     */
-//     const
-//     PlayerObject * getOpponent( const int unum ) const
-//       {
-//           const PlayerPtrCont::const_iterator end = opponentsFromSelf().end();
-//           for ( PlayerPtrCont::const_iterator it = opponentsFromSelf().begin();
-//                 it != end;
-//                 ++it )
-//           {
-//               if ( (*it)->unum() == unum )
-//               {
-//                   return *it;
-//               }
-//           }
-//           return static_cast< PlayerObject * >( 0 );
-//       }
     const
     AbstractPlayerObject * opponent( const int unum ) const
       {
@@ -1120,6 +1095,13 @@ public:
      */
     AbstractPlayerCont getPlayerCont( const PlayerPredicate * predicate ) const;
 
+    /*!
+      \brief get the new container of AbstractPlayer matched with the predicate.
+      \param predicate predicate object for the player condition matching.
+      \return container of AbstractPlayer pointer.
+     */
+    AbstractPlayerCont getPlayerCont( boost::shared_ptr< const PlayerPredicate > predicate ) const;
+
     /*!
       \brief get the new container of AbstractPlayer matched with the predicate.
       \param cont reference to the result variable
@@ -1128,6 +1110,28 @@ public:
     void getPlayerCont( AbstractPlayerCont & cont,
                         const PlayerPredicate * predicate ) const;
 
+    /*!
+      \brief get the new container of AbstractPlayer matched with the predicate.
+      \param cont reference to the result variable
+      \param predicate predicate object for the player condition matching.
+     */
+    void getPlayerCont( AbstractPlayerCont & cont,
+                        boost::shared_ptr< const PlayerPredicate > predicate ) const;
+
+    /*!
+      \brief get the number of players that satisfy an input predicate.
+      \param predicate predicate predicate object for the player condition matching. This have to be a dynamically allocated object.
+      \return number of players.
+     */
+    int countPlayer( const PlayerPredicate * predicate ) const;
+
+    /*!
+      \brief get the number of players that satisfy an input predicate.
+      \param predicate predicate predicate object for the player condition matching.
+      \return number of players.
+     */
+    int countPlayer( boost::shared_ptr< const PlayerPredicate > predicate ) const;
+
     /*!
       \brief get teammate goalie pointer
       \return if found pointer to goalie object, otherwise NULL
