@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -37,6 +37,7 @@
 
 #include "reader.h"
 #include "parser_v4.h"
+#include "parser_v5.h"
 
 #ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
@@ -131,5 +132,20 @@ Holder::addDisp3( const std::string & msg )
     return parser.parseLine( 0, msg, reader );
 }
 
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+bool
+Holder::addDisp4( const std::string & msg )
+{
+    setLogVersion( REC_VERSION_5 );
+
+    Reader reader( *this );
+    ParserV5 parser;
+
+    return parser.parseLine( 0, msg, reader );
+}
+
 } // end namespace
 } // end namespace
