@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,14 +35,14 @@
 
 #include "parser.h"
 
+#include "types.h"
+
 #include "parser_v1.h"
 #include "parser_v2.h"
 #include "parser_v3.h"
 #include "parser_v4.h"
 #include "parser_v5.h"
 
-#include "types.h"
-
 namespace rcsc {
 namespace rcg {
 
@@ -50,10 +50,10 @@ namespace rcg {
 /*!
 
 */
-Parser::FactoryHolder &
-Parser::factoryHolder()
+Parser::Creators &
+Parser::creators()
 {
-    static FactoryHolder s_instance;
+    static Creators s_instance;
     return s_instance;
 }
 
@@ -65,9 +65,10 @@ Parser::factoryHolder()
 Parser::Ptr
 Parser::create( std::istream & is )
 {
+    Parser::Ptr ptr( static_cast< Parser * >( 0 ) );
+
     char header[5];
     int version = REC_OLD_VERSION;
-    Parser::Ptr ptr;
 
     is.read( header, 4 ); // read 'U', 'L', 'G', <version>
 
@@ -84,7 +85,7 @@ Parser::create( std::istream & is )
     }
 
     Parser::Creator creator;
-    if ( Parser::factoryHolder().getCreator( creator, version ) )
+    if ( Parser::creators().getCreator( creator, version ) )
     {
         ptr = creator();
     }
