@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -309,5 +309,23 @@ ParserV3::parsePlayerParam( std::istream & is,
     return false;
 }
 
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+namespace {
+
+Parser::Ptr
+create_v3()
+{
+    Parser::Ptr ptr( new ParserV3() );
+    return ptr;
+}
+
+rcss::RegHolder v3 = Parser::creators().autoReg( &create_v3, REC_VERSION_3 );
+
+}
+
 } // end of namespace
 } // end of namespace
