@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -320,5 +320,23 @@ SerializerV1::serialize( std::ostream & os,
     return serialize( os, disp.show_ );
 }
 
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+namespace {
+
+Serializer::Ptr
+create_v1()
+{
+    Serializer::Ptr ptr( new SerializerV1() );
+    return ptr;
+}
+
+rcss::RegHolder v1 = Serializer::creators().autoReg( &create_v1, REC_OLD_VERSION );
+
+}
+
 } // end of namespace
 } // end of namespace
