@@ -124,7 +124,7 @@ public:
     bool handleShowInfo( const rcsc::rcg::showinfo_t & show );
     bool handleShortShowInfo2( const rcsc::rcg::short_showinfo_t2 & show );
     bool handleMsgInfo( rcsc::rcg::Int16,
-                        const std::string & msg )
+                        const std::string & )
       {
           return true;
       }
@@ -143,7 +143,7 @@ public:
                      const rcsc::rcg::ShowInfoT & show );
     bool handleMsg( const int time,
                     const int board,
-                    const char * msg );
+                    const std::string & msg );
     bool handlePlayMode( const int time,
                          const rcsc::PlayMode pm );
     bool handleTeam( const int time,
@@ -384,7 +384,7 @@ TextPrinter::handleTeamInfo( const rcsc::rcg::team_t & team_left,
 
  */
 bool
-TextPrinter::handlePlayerType( const rcsc::rcg::player_type_t & param )
+TextPrinter::handlePlayerType( const rcsc::rcg::player_type_t & )
 {
     std::cerr << "handlePlayerType" << std::endl;
     return true;
@@ -425,7 +425,7 @@ TextPrinter::handleServerParam( const rcsc::rcg::server_params_t & param )
 
  */
 bool
-TextPrinter::handlePlayerParam( const rcsc::rcg::player_params_t & param )
+TextPrinter::handlePlayerParam( const rcsc::rcg::player_params_t & )
 {
     std::cerr << "handlePlayerParam" << std::endl;
     return true;
@@ -452,7 +452,7 @@ TextPrinter::handleEOF()
 
  */
 bool
-TextPrinter::handleShow( const int time,
+TextPrinter::handleShow( const int,
                          const rcsc::rcg::ShowInfoT & show )
 {
     if ( ! M_init_written )
@@ -492,9 +492,9 @@ TextPrinter::handleShow( const int time,
 
  */
 bool
-TextPrinter::handleMsg( const int time,
-                        const int board,
-                        const char * msg )
+TextPrinter::handleMsg( const int,
+                        const int,
+                        const std::string & )
 {
     return true;
 }
@@ -504,7 +504,7 @@ TextPrinter::handleMsg( const int time,
 
  */
 bool
-TextPrinter::handlePlayMode( const int time,
+TextPrinter::handlePlayMode( const int,
                              const rcsc::PlayMode pm )
 {
     M_playmode = pm;
@@ -516,7 +516,7 @@ TextPrinter::handlePlayMode( const int time,
 
  */
 bool
-TextPrinter::handleTeam( const int time,
+TextPrinter::handleTeam( const int,
                          const rcsc::rcg::TeamT & team_l,
                          const rcsc::rcg::TeamT & team_r )
 {
@@ -558,7 +558,7 @@ TextPrinter::handleServerParam( const std::string & msg )
 
  */
 bool
-TextPrinter::handlePlayerParam( const std::string & msg )
+TextPrinter::handlePlayerParam( const std::string & )
 {
     return true;
 }
@@ -568,7 +568,7 @@ TextPrinter::handlePlayerParam( const std::string & msg )
 
  */
 bool
-TextPrinter::handlePlayerType( const std::string & msg )
+TextPrinter::handlePlayerType( const std::string & )
 {
     return true;
 }
@@ -872,8 +872,8 @@ int
 main( int argc, char** argv )
 {
     if ( argc != 2
-         || ! strcmp( argv[1], "--help" )
-         || ! strcmp( argv[1], "-h" ) )
+         || ! std::strncmp( argv[1], "--help", 6 )
+         || ! std::strncmp( argv[1], "-h", 2 ) )
     {
         std::cerr << "usage: " << argv[0] << " <RcgFile>[.gz]" << std::endl;
         return 0;
@@ -887,7 +887,7 @@ main( int argc, char** argv )
         return 1;
     }
 
-    rcsc::rcg::ParserPtr parser = rcsc::rcg::make_parser( fin );
+    rcsc::rcg::Parser::Ptr parser = rcsc::rcg::Parser::create( fin );
 
     if ( ! parser )
     {
