@@ -24,7 +24,7 @@ private:
 
     std::ostream & M_os;
 
-    rcsc::rcg::SerializerPtr M_serializer;
+    rcsc::rcg::Serializer::Ptr M_serializer;
 
     // not used
     Reverser();
@@ -55,7 +55,7 @@ public:
                      const rcsc::rcg::ShowInfoT & show );
     bool handleMsg( const int time,
                     const int board,
-                    const char * msg );
+                    const std::string & msg );
     bool handlePlayMode( const int time,
                          const rcsc::PlayMode pm );
     bool handleTeam( const int time,
@@ -84,7 +84,7 @@ private:
 Reverser::Reverser( std::ostream & os )
     : M_os( os )
 {
-    M_serializer = rcsc::rcg::make_serializer( 1 );
+    M_serializer = rcsc::rcg::Serializer::create( 1 );
 }
 
 /*-------------------------------------------------------------------*/
@@ -96,7 +96,7 @@ Reverser::handleLogVersion( const int ver )
 {
     rcsc::rcg::Handler::handleLogVersion( ver );
 
-    M_serializer = rcsc::rcg::make_serializer( ver );
+    M_serializer = rcsc::rcg::Serializer::create( ver );
 
     if ( ! M_serializer )
     {
@@ -324,7 +324,7 @@ Reverser::handleEOF()
 
 */
 bool
-Reverser::handleShow( const int time,
+Reverser::handleShow( const int,
                       const rcsc::rcg::ShowInfoT & show )
 {
     if ( ! M_serializer )
@@ -356,9 +356,9 @@ Reverser::handleShow( const int time,
 
 */
 bool
-Reverser::handleMsg( const int time,
+Reverser::handleMsg( const int,
                      const int board,
-                     const char * msg )
+                     const std::string & msg )
 {
     if ( ! M_serializer )
     {
@@ -374,7 +374,7 @@ Reverser::handleMsg( const int time,
 
 */
 bool
-Reverser::handlePlayMode( const int time,
+Reverser::handlePlayMode( const int,
                           const rcsc::PlayMode pm )
 {
     if ( ! M_serializer )
@@ -391,7 +391,7 @@ Reverser::handlePlayMode( const int time,
 
 */
 bool
-Reverser::handleTeam( const int time,
+Reverser::handleTeam( const int,
                       const rcsc::rcg::TeamT & team_l,
                       const rcsc::rcg::TeamT & team_r )
 {
@@ -637,7 +637,7 @@ main( int argc, char** argv )
         return 1;
     }
 
-    rcsc::rcg::ParserPtr parser = rcsc::rcg::make_parser( fin );
+    rcsc::rcg::Parser::Ptr parser = rcsc::rcg::Parser::create( fin );
 
     if ( ! parser )
     {
